/*******************************************************************************

  AMXMODX_TO_AMX Library
  Author: KRoT@L
  Version: 1.1.0

  You need AMX 2006.
  
  Credit for the original AmxModX natives/stocks/defines goes to AmxModX Dev Team.

*******************************************************************************/

/*******************************************************************************
* INSTRUCTIONS
********************************************************************************

Put this file inside your amx/examples/include folder.

--------------------------------------------------------------------------------

Replace all the #include <...> lines with just this line:
#include <amxmodx_to_amx>

For example, replace this:

#include <amxmodx>
#include <amxmisc>
#include <engine>
#include <fun>

with just

#include <amxmodx_to_amx>

--------------------------------------------------------------------------------

If the plugin uses csstats, add:
#include <csstats>
BEFORE #include <amxmodx_to_amx>

If the plugin uses dodxmod, add:
#include <dodxmod>
BEFORE #include <amxmodx_to_amx>

--------------------------------------------------------------------------------

Other changes you may need to do:

  Replace

public pfn_touch(ptr, ptd) {

  with

public entity_touch(entity1, entity2) {
  new ptr = entity1
  new ptd = entity2

*******************************************************************************/

/*******************************************************************************
*  This program is free software; you can redistribute it and/or modify it
*  under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 2 of the License, or (at
*  your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software Foundation,
*  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*  In addition, as a special exception, the author gives permission to
*  link the code of this program with the Half-Life Game Engine ("HL
*  Engine") and Modified Game Libraries ("MODs") developed by Valve,
*  L.L.C ("Valve"). You must obey the GNU General Public License in all
*  respects for all of the code used other than the HL Engine and MODs
*  from Valve. If you modify this file, you may extend this exception
*  to your version of the file, but you are not obligated to do so. If
*  you do not wish to do so, delete this exception statement from your
*  version.
*******************************************************************************/

/*******************************************************************************
* INCLUDES
*******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>
#include <cstrike>
#include <fun>
#include <geoip>
#include <misc_stocks>
#include <sockets>
#include <Vexd_Utilities>
#include <VexdUM_stock>

/*******************************************************************************
* DEFINES
*******************************************************************************/

#define AMXX_VERSION      2006
#define AMXX_VERSION_NUM  2006
#define AMXX_VERSION_STR  AMX_VERSION

#define ADMIN_ALL         0   /* everyone */
#define ADMIN_ADMIN (1<<24)   /* flag "y" */

#define CSW_VEST        31
#define CSW_VESTHELM    32

#define MENU_KEY_1      (1<<0)
#define MENU_KEY_2      (1<<1)
#define MENU_KEY_3      (1<<2)
#define MENU_KEY_4      (1<<3)
#define MENU_KEY_5      (1<<4)
#define MENU_KEY_6      (1<<5)
#define MENU_KEY_7      (1<<6)
#define MENU_KEY_8      (1<<7)
#define MENU_KEY_9      (1<<8)
#define MENU_KEY_0      (1<<9)

#define LANG_SERVER     0
#define LANG_PLAYER    -1

#define FMRES_HANDLED   PLUGIN_CONTINUE
#define FMRES_SUPERCEDE PLUGIN_HANDLED
#define FMRES_IGNORED   PLUGIN_CONTINUE
#define FMRES_OVERRIDE  PLUGIN_CONTINUE

#define STAMINA_SET     0
#define STAMINA_RESET   1

#define FUSE_SET        0
#define FUSE_RESET      1

#define PS_NOPRONE      0
#define PS_PRONE        1
#define PS_PRONEDEPLOY  2
#define PS_DEPLOY       3

#define MI_ALLIES_TEAM  0
#define MI_ALLIES_PARAS 1
#define MI_AXIS_PARAS   2

#define M_PI 3.1415926535897932384626433832795

/*******************************************************************************
* AMXMISC.INC
*******************************************************************************/

stock cstrike_running() {
  new mod_name[32];
  get_modname(mod_name,31);
  return (equal(mod_name,"cstrike") || equal(mod_name,"czero"));
}

stock get_configsdir(name[],len) {
  return build_path(name,len,"$configdir");
}

stock get_datadir(name[],len) {
  return build_path(name,len,"$modulesdir");
}

stock get_customdir(name[],len) {
  return build_path(name,len,"$configdir");
}

stock register_menu(title[],keys,function[],outside=0) {
  register_menucmd(register_menuid(title,outside),keys,function);
}

stock AddMenuItem(const MENU_TEXT[], const MENU_CMD[], const MENU_ACCESS, const MENU_PLUGIN[]) {
  new szAccess[16];
  get_flags(MENU_ACCESS, szAccess, 15);
  server_cmd("amx_addmenuitem ^"amxmodmenu^" ^"%s^" ^"%s^" ^"%s^" ^"b^" ^"%s^"",MENU_PLUGIN,MENU_TEXT,MENU_CMD,szAccess);
}

stock constraint_offset(low, high, seed, offset) {
  new numElements = high - low + 1;
  offset += seed - low;
  if(offset >= 0)
    return low + (offset % numElements);
  else
    return high - (abs(offset) % numElements) + 1;
  return 0;
}

/*******************************************************************************
* AMXMODX.INC
*******************************************************************************/

stock user_has_weapon(index,weapon,setweapon=-1) {
  if(setweapon == 0 || setweapon == 1) {
    return set_user_weapon(index, weapon, setweapon);
  }
  return has_user_weapon(index, weapon);
}

stock log_amx(string[], {Float,_}:...) {
  if(string[0]) {}
  new str[512];
  format_args(str, 511, 0);
  new logfile[16];
  get_logfile(logfile, 15);
  return log_to_file(logfile, str);
}

stock change_task(id = 0, Float:newTime=1.0, outside = 0) {
  return change_tasktime(id, new_time, outside);
}

stock is_module_loaded(const name[]) {
  return is_module_running(name);
}

stock is_plugin_loaded(const name[]) {
  return is_plugin_running(name);
}

stock require_module(const module[]) {
  return module[0];
}

stock is_amd64_server() {
  return 0;
}

stock find_plugin_byfile(const filename[], ignoreCase=1) {
  return is_plugin_running(filename);
}

stock log_error(error, const fmt[], ...) {
  new str[512];
  format_args(str, 511, 1);
  return report_error(str);
}

stock abort(error, const fmt[]="", {Float,_}:...) {
  new str[512];
  format_args(str, 511, 1);
  return report_error(str);
}

stock module_exists(const logtag[]) {
  return is_module_running(logtag);
}

/*******************************************************************************
* DODX.INC
*******************************************************************************/

#define client_damage(attacker,victim,damage,wpnindex,hitplace,TA) \
  damage_info(attacker,victim,damage,wpnindex,hitplace,TA)

#define client_death(killer,victim,wpnindex,hitplace,TK) \
  death_info(killer,victim,wpnindex,hitplace,TK)

#define client_score(index,score,total) \
  score_info(index,score,total)

enum {
  XMF_DAMAGE = 0,
  XMF_DEATH,
  XMF_SCORE,
};

stock register_statsfwd(ftype) {
  return ftype;
}

stock dod_get_map_info(info) {
  switch(info) {
    case 0: return detect_allies_team();
    case 1: return detect_allies_paras();
    case 2: return detect_axis_paras();
  }
  return 0;
}

stock xmod_is_melee_wpn(wpnindex) {
  return dod_is_melee(wpnindex);
}

stock xmod_get_wpnname(wpnindex,name[],len) {
  return dod_get_wpnname(wpnindex,name,len);
}

stock xmod_get_wpnlogname(wpnindex,name[],len) {
  return dod_get_wpnlogname(wpnindex,name,len);
}

/*******************************************************************************
* DODFUN.INC
*******************************************************************************/

stock dod_set_stamina(index,set=0,minvalue=0,maxvalue=100) {
  if(set == 0) {
    return set_user_stamina(index,minvalue,maxvalue);
  }
  return reset_user_stamina(index);
}

stock dod_set_fuse(index,set=0,Float:newFuse=5.0,Type=(1<<0)) {
  if(set == 0) {
    return nade_set_fuse(index,newFuse,Type);
  }
  return nade_reset_fuse(index);
}

/*******************************************************************************
* ENGINE.INC
*******************************************************************************/

stock force_use(pPlayer, pEntity) {
  return entity_use(pEntity, pPlayer);
}

stock Float:get_global_float(variable) {
  return global_get_float(variable);
}

stock get_global_int(variable) {
  return global_get_int(variable);
}

stock get_global_string(variable, string[], maxlen) {
  return global_get_string(variable, string, maxlen);
}

stock get_global_vector(variable, Float:vector[3]) {
  return global_get_vector(variable, vector);
}

stock get_global_edict(variable) {
  return global_get_edict(variable);
}

stock Float:entity_range(ida,idb) {
  new Float:orig1[3], Float:orig2[3];
  entity_get_vector(ida, EV_VEC_origin, orig1);
  entity_get_vector(idb, EV_VEC_origin, orig2);
  return vector_distance(orig1, orig2);
}

stock find_ent_by_class(iIndex, szClass[]) {
  if(iIndex == 0) iIndex = -1;
  new ent = find_entity(iIndex, szClass);
  return (ent > 0) ? ent : 0;
}

stock find_ent_by_owner(iIndex, szClass[], iOwner, iJghgType=0) {
  new maxEntities = get_maxentities();
  for(new i = iIndex+1 ; i <= maxEntities; i++) {
    if(!is_entity(i)) continue;
    if(entity_get_edict(i, EV_ENT_owner) == iOwner) {
      switch(iJghgType) {
        case 0: {
          new classname[64];
          entity_get_string(i, EV_SZ_classname, classname, 63);
          if(equal(szClass, classname)) {
            return i;
          }
        }
        case 1: {
          new target[64];
          entity_get_string(i, EV_SZ_target, target, 63);
          if(equal(szClass, target)) {
            return i;
          }
        }
        case 2: {
          new targetname[64];
          entity_get_string(i, EV_SZ_targetname, targetname, 63);
          if(equal(szClass, targetname)) {
            return i;
          }
        }
      }
    }
  }
  return 0;
}

stock find_ent_by_target(iIndex, szClass[]) {
  new maxEntities = get_maxentities();
  for(new i = iIndex+1 ; i <= maxEntities; i++) {
    if(!is_entity(i)) continue;
    new target[64];
    entity_get_string(i, EV_SZ_target, target, 63);
    if(equal(szClass, target)) {
      return i;
    }
  }
  return 0;
}

stock find_ent_by_tname(iIndex, szClass[]) {
  new maxEntities = get_maxentities();
  for(new i = iIndex+1 ; i <= maxEntities; i++) {
    if(!is_entity(i)) continue;
    new targetname[64];
    entity_get_string(i, EV_SZ_targetname, targetname, 63);
    if(equal(szClass, targetname)) {
      return i;
    }
  }
  return 0;
}

stock find_ent_by_model(iIndex, szClass[], szModel[]) {
  if(iIndex == 0) iIndex = -1;
  new ent = find_entity(iIndex, szClass);
  while(ent > 0) {
    new model[64];
    entity_get_string(ent, EV_SZ_model, model, 63);
    if(equal(szModel, model)) {
      return ent;
    }
    ent = find_entity(ent, szClass);
  }
  return 0;
}

stock find_ent_in_sphere(start_from_ent, Float:origin[3], Float:radius) {
  if(start_from_ent == 0) start_from_ent = -1;
  new ent = find_entity_sphere(start_from_ent, origin, radius)
  return (ent > 0) ? ent : 0;
}

stock is_valid_ent(iIndex) {
  return is_entity(iIndex);
}

stock entity_count() {
  return get_num_ents();
}

stock velocity_by_aim(iIndex, iVelocity, Float:vRetValue[3]) {
  return VelocityByAim(iIndex, iVelocity, vRetValue);
}

stock point_contents(Float:fCheckAt[3]) {
  return PointContents(fCheckAt);
}

stock angle_vector(Float:angle[3],FRU,Float:ret[3]) {
  return angle_to_vector(angle, FRU, ret);
}

stock Float:halflife_time() {
  return get_gametime();
}

stock find_sphere_class(aroundent,_lookforclassname[],Float:radius,entlist[],maxents,Float:origin[3]={0.0,0.0,0.0}) {
  if(is_entity(aroundent)) {
    entity_get_vector(aroundent, EV_VEC_origin, origin);
  }
  new entnum = 0;
  new ent = find_entity_sphere(-1, origin, radius);
  while(ent > 0 && entnum < maxents) {
    new classname[64];
    entity_get_string(ent, EV_SZ_classname, classname, 63);
    if(equal(classname, _lookforclassname)) {
      entlist[entnum++] = ent;
    }
    ent = find_entity_sphere(ent, origin, radius);
  }
  return entnum;
}

stock is_in_viewcone(entity, Float:origin[3]) {
  return in_view_cone(entity, origin);
}

stock is_visible(entity, target) {
  return can_see(entity, target);
}

#define client_PreThink(id) client_prethink(id)
#define client_PostThink(id) client_postthink(id)
#define pfn_think(entid) entity_think(entity)
#define pfn_spawn(entid) entity_spawn(entity)

/*******************************************************************************
* ENGINE_STOCKS.INC
*******************************************************************************/

stock fakedamage(idvictim,szClassname[],Float:takedmgdamage,damagetype) {
  new entity = create_entity("trigger_hurt");
  if(entity > 0) {
    DispatchKeyValue(entity,"classname","trigger_hurt");
    new szDamage[16];
    format(szDamage,15,"%f",takedmgdamage * 2);
    DispatchKeyValue(entity,"dmg",szDamage);
    format(szDamage,15,"%i",damagetype);
    DispatchKeyValue(entity,"damagetype",szDamage);
    DispatchKeyValue(entity,"origin","8192 8192 8192");
    DispatchSpawn(entity);
    entity_set_string(entity, EV_SZ_classname, szClassname);
    fake_touch(entity,idvictim);
    remove_entity(entity);
    return 1;
  }
  return 0;
}

stock find_ent(iStart, szClassname[]) {
  new ent = find_entity(iStart, szClassname)
  return (ent > 0) ? ent : 0;
}

stock remove_entity_name(eName[]) {
  return remove_entities(eName[]);
}

stock make_deathmsg(killer,victim,headshot,weapon[]) {
  message_begin(MSG_ALL,get_user_msgid("DeathMsg"),{0,0,0},0);
  write_byte(killer);
  write_byte(victim);
  new mod_name[32];
  get_modname(mod_name,31);
  if(equal(mod_name,"cstrike") || equal(mod_name,"czero")) {
    write_byte(headshot);
  }
  write_string(weapon);
  message_end();
  return 1;
}

stock dod_make_deathmsg(killer,victim,weaponNUM) {
  message_begin(MSG_ALL,get_user_msgid("DeathMsg"),{0,0,0},0);
  write_byte(killer);
  write_byte(victim);
  write_byte(weaponNUM);
  message_end();
  return 1;
}

stock user_silentkill(index) {
  set_msg_block(get_user_msgid("DeathMsg"),BLOCK_ONCE);
  user_kill(index,1);
  return 1;
}

stock set_entity_visibility(entity, visible = 1) {
  if(visible == 1) {
    entity_set_int(entity, EV_INT_effects, entity_get_int(entity, EV_INT_effects) & ~EF_NODRAW);
  }
  else {
    entity_set_int(entity, EV_INT_effects, entity_get_int(entity, EV_INT_effects) | EF_NODRAW);
  }
  return 1;
}

stock get_entity_visibility(entity) {
  return (entity_get_int(entity, EV_INT_effects) & EF_NODRAW);
}

stock set_user_velocity(entity, Float:vec[3]) {
  return entity_set_vector(entity, EV_VEC_velocity, vec);
}

stock get_user_velocity(entity, Float:vec[3]) {
  return entity_get_vector(entity, EV_VEC_velocity, vec);
}

/*******************************************************************************
* FAKEMETA.INC
*******************************************************************************/

stock get_pdata_int(_index,_Offset,_linuxdiff=5) {
  return get_offset_int(_index,_Offset,_linuxdiff);
}

stock set_pdata_int(_index,_Offset,_Value,_linuxdiff=5) {
  return set_offset_int(_index,_Offset,_Value,_linuxdiff);
}

/*******************************************************************************
* FILE.INC
*******************************************************************************/

stock filesize(const filename[512],{Float,_}:...) {
  new str[512];
  format_args(str, 511, 0);
  return file_size(str);
}

stock unlink(const filename[512],{Float,_}:...) {
  new str[512];
  format_args(str, 511, 0);
  return delete_file(str);
}

/*******************************************************************************
* FLOAT.INC
*******************************************************************************/

stock Float:floatabs(Float:value) {
  return fabs(value);
}

enum anglemode {
  radian = 0,
  degrees,
  grades
};

stock Float:FromRadians(Float:angle, radix) {
  switch(radix) {
    case 1: return (angle / PI * 180.0);
    case 2: return (angle / PI * 200.0);
  }
  return angle;
}

stock Float:ToRadians(Float:angle, radix) {
  switch(radix) {
    case 1: return (angle * PI / 180.0);
    case 2: return (angle * PI / 200.0);
  }
  return angle;
}

stock Float:floatsin(Float:value, anglemode:mode=radian) {
  value = ToRadians(value, mode);
  return sin(value);
}

stock Float:floatcos(Float:value, anglemode:mode=radian) {
  value = ToRadians(value, mode);
  return cos(value);
}

stock Float:floattan(Float:value, anglemode:mode=radian) {
  value = ToRadians(value, mode);
  return tan(value);
}

stock Float:floatatan(Float:angle, radix) {
  angle = ToRadians(angle, radix);
  angle = atan(angle);
  angle = FromRadians(angle, radix);
  return angle;
}

stock Float:floatacos(Float:angle, radix) {
  angle = acos(angle);
  angle = FromRadians(angle, radix);
  return angle;
}

stock Float:floatasin(Float:angle, radix) {
  angle = asin(angle);
  angle = FromRadians(angle, radix);
  return angle;
}

stock Float:floatatan2(Float:x, Float:y, radix) {
  new Float:z;
  z = atan2(x, y);
  z = FromRadians(z, radix);
  return z;
}

stock Float:floatsqroot(Float:value) {
  return fsqroot(value)
}

stock Float:floatpower(Float:value, Float:exponent) {
  return fpower(value, exponent)
}

stock Float:floatlog(Float:value, Float:base=10.0) {
  return flog(value, base)
}

/*******************************************************************************
* FUN.INC
*******************************************************************************/

stock spawn(index) {
  return user_spawn(index);
}

/*******************************************************************************
* LANG.INC
*******************************************************************************/

stock get_langsnum() {
  return get_languagesnum();
}

stock get_lang(id, name[3]) {
  return get_language(id, name, 2);
}

stock lang_exists(const name[]) {
  return is_language_known(name);
}

/*******************************************************************************
* STRING.INC
*******************************************************************************/

stock float_to_str(Float:fl, string[], len) {
  return format(string, len, "%f", fl);
}

stock Float:str_to_float(const string[]) {
  return floatstr(string);
}

stock strcat(dest[], const source[], maxlength) {
  return add(dest, maxlength, source);
}

stock strfind(const string[], const sub[], ignorecase=0, pos=0) {
  if(pos >= strlen(string)) return -1;
  if(ignorecase) {
    return containi(string[pos], sub);
  }
  return contain(string[pos], sub);
}

stock strcmp(const string1[], const string2[], ignorecase=0) {
  if(ignorecase) {
    return (equali(string1, string2)) ? 0 : -1;
  }
  return (equal(string1, string2)) ? 0 : -1;
}

stock ucfirst(string[]) {
  string[0] = toupper(string[0]);
  return 1;
}

stock remove_filepath(szFilePath[], szFile[], pMax) {
	new len = strlen(szFilePath);
	while((--len >= 0) && (szFilePath[len] != '/') && (szFilePath[len] != '\')) { }
	copy(szFile, pMax, szFilePath[len + 1]);
	return 1;
}

/*******************************************************************************
* XTRAFUN.INC
*******************************************************************************/

stock get_entity_velocity(id, velocity[3]) {
  new Float:vel[3];
  entity_get_vector(id, EV_VEC_velocity, vel);
  FVecIVec(vel, velocity);
}

stock set_entity_velocity(id, velocity[3]) {
  new Float:vel[3];
  IVecFVec(velocity, vel);
  entity_set_vector(id, EV_VEC_velocity, vel);
}

stock get_entity_origin(id, origin[3]) {
  new Float:orig[3];
  entity_get_vector(id, EV_VEC_origin, orig);
  FVecIVec(orig, origin);
}

stock set_entity_origin(id, origin[3]) {
  new Float:orig[3];
  IVecFVec(origin, orig);
  entity_set_vector(id, EV_VEC_origin, orig);
}

stock get_grenade_id(id, model[], len, grenadeid) {
  grenadeid = get_grenade(id, grenadeid);
  entity_get_string(grenadeid, EV_SZ_model, model, len);
  return grenadeid;
}

stock get_grenade_index(index, model[], len, grenadeindex = 0) {
  if(grenadeindex == 0) grenadeindex = -1;
  new entfind = grenadeindex;
  new entowner = index;
  for(;;) {
    entfind = find_entity(entfind, "grenade");
    if(entfind > 0 && is_entity(entfind)) {
      if(entity_get_edict(entFind, EV_ENT_owner) == entowner) {
        entity_get_string(entfind, EV_SZ_model, model, len);
        return entfind;
      }
    }
    else {
      return 0;
    }
  }
}

stock current_num_ents() {
  return get_num_ents();
}
