/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

#include <translator>
#include <amxmod>
#include <amxmisc>

// Uncomment the following line to load admins from a database
//#define USE_SQL

/* If you enable USE_SQL, you might create your MySQL admins table with this SQL query :

  CREATE TABLE admins (
    `auth` varchar(32) NOT NULL default '',
    `password` varchar(32) NOT NULL default '',
    `access` varchar(32) NOT NULL default '',
    `flags` varchar(32) NOT NULL default '',
    PRIMARY KEY (`auth`)
  ) TYPE=MyISAM;

* IMPORTANT:
* Check $moddir/addons/amx/config/mysql.cfg for MySQL access configuration
*
*/

#define MAX_ADMINS 64

new g_aPassword[MAX_ADMINS][32]
new g_aName[MAX_ADMINS][32]
new g_aFlags[MAX_ADMINS]
new g_aAccess[MAX_ADMINS]
new g_aNum
new g_whoGaveTempAdmin[33]
new g_logFile[16]
new g_cmdLoopback[16]
new g_maxplayers

#if defined USE_SQL
#include <mysql>
#endif

public plugin_init() {
  new filename[64]
  build_path(filename, 63, "$configdir")
  server_cmd("exec %s/amx.cfg", filename)
  server_exec()
  
  load_translations("adminbase")
#if defined USE_SQL
  register_plugin(_T("Admin Base for MySQL"),AMX_VERSION,"AMX Team")
#else
  register_plugin(_T("Admin Base"),AMX_VERSION,"AMX Team")
#endif

  register_concmd("amx_addadmin","cmdAddAdmin",ADMIN_RCON,_T("<name|#userid|authid|ip> <password> <access> <flags>"))
  register_concmd("amx_removeadmin","cmdRemoveAdmin",ADMIN_RCON,_T("<name|#userid|authid|ip>"))
  register_concmd("amx_makeadmin","cmdMakeAdmin",ADMIN_RCON,_T("<name|#userid|authid> [accessflags] - give admin powers to player till disconnect/mapchange"))

  register_cvar("amx_mode","1")
  register_cvar("amx_password_field","_pw")
  register_cvar("amx_default_access","z")
  register_cvar("amx_correct_usersfile","0")

  register_cvar("amx_votekick_ratio","0.40")
  register_cvar("amx_voteban_ratio","0.40")
  register_cvar("amx_votemap_ratio","0.40")
  register_cvar("amx_vote_ratio","0.50")
  register_cvar("amx_vote_time","10")
  register_cvar("amx_vote_answers","1")
  register_cvar("amx_vote_delay","10")
  register_cvar("amx_last_voting","0")
  register_cvar("amx_show_activity","2")
  set_cvar_float("amx_last_voting",0.0)

  register_cvar("amx_mysql_host","127.0.0.1")
  register_cvar("amx_mysql_user","root")
  register_cvar("amx_mysql_pass","")
  register_cvar("amx_mysql_db","amx")

  register_srvcmd("amx_loadadmins","loadAdmin")
#if defined USE_SQL
  register_srvcmd("amx_loadsqladmins","loadAdminSql")
#endif

  get_logfile(g_logFile, 15)
  new mapname[32]
  get_mapname(mapname, 31)
  log_to_file(g_logFile,"********** Loaded map %s **********",mapname)

  format(g_cmdLoopback, 15, "amxauth%c%c%c%c", random_num('A','Z'), random_num('A','Z'), random_num('A','Z'), random_num('A','Z'))
  register_clcmd(g_cmdLoopback,"ackSignal")
  
  g_maxplayers = get_maxplayers()
}

public plugin_cfg()
{
  new filename[64]
  build_path(filename, 63, "$configdir")
  server_cmd("exec %s/amx.cfg", filename) // Execute main configuration file
  server_exec()
  server_cmd("exec %s/mysql.cfg", filename) // Execute sql configuration file
  server_exec()
#if defined USE_SQL
  server_cmd("amx_loadsqladmins")
#else
  add(filename, 63, "/users.ini")
  loadSettings(filename, get_cvar_num("amx_correct_usersfile")) // Load admins accounts
#endif
}

public loadAdmin() {
  new filename[64]
  build_path(filename, 63, "$configdir/users.ini")
  loadSettings(filename, get_cvar_num("amx_correct_usersfile"))
  return PLUGIN_HANDLED
}

#if defined USE_SQL
public loadAdminSql() {
  remove_user_flags(0, read_flags("z"))
  
  new host[64], user[32], pass[32], db[32], error[128]
  get_cvar_string("amx_mysql_host", host, 63)
  get_cvar_string("amx_mysql_user", user, 31)
  get_cvar_string("amx_mysql_pass", pass, 31)
  get_cvar_string("amx_mysql_db", db, 31)
  
  new filename[64]
  build_path(filename, 63, "$configdir/sql_users.ini")
  new mysql = mysql_connect(host, user, pass, db, error, 127)
  if(mysql < 1) {
    log_to_file(g_logFile,_T("MySQL error: can't connect: '%s'"),error)
    log_to_file(g_logFile,_T("Loading admins from sql_users.ini"))
    if(!loadSettings(filename, 0)) {
      log_to_file(g_logFile,_T("Loading admins from users.ini"))
      build_path(filename, 63, "$configdir/users.ini", filename)
      loadSettings(filename, get_cvar_num("amx_correct_usersfile"))
    }
    return PLUGIN_HANDLED 
  }

  if(mysql_query(mysql, "SELECT auth,password,access,flags FROM admins") < 1) {
    mysql_error(mysql, error, 127)
    log_to_file(g_logFile,_T("MySQL error: can't load admins: '%s'"),error)
    mysql_close(mysql)
    log_to_file(g_logFile,_T("Loading admins from sql_users.ini"))
    if(!loadSettings(filename, 0)) {
      log_to_file(g_logFile,_T("Loading admins from users.ini"))
      build_path(filename, 63, "$configdir/users.ini", filename)
      loadSettings(filename, get_cvar_num("amx_correct_usersfile"))
    }
    return PLUGIN_HANDLED
  }

  if(file_exists(filename)) delete_file(filename)
  write_file(filename, "; Generated by Admin Base for MySQL Plugin. Do not modify! Modify your database instead.^n^n")

  new szFlags[32], szAccess[32]
  g_aNum = 0
  while(g_aNum < MAX_ADMINS && mysql_nextrow(mysql) > 0) {
    mysql_getfield(mysql, 1, g_aName[g_aNum], 31)
    mysql_getfield(mysql, 2, g_aPassword[g_aNum], 31)
    mysql_getfield(mysql, 3, szAccess, 31)
    mysql_getfield(mysql, 4, szFlags, 31)
    g_aAccess[g_aNum] = read_flags(szAccess)
    g_aFlags[g_aNum] = read_flags(szFlags)
    new lineToAdd[256]
    format(lineToAdd, 255, "^"%s^" ^"%s^" ^"%s^" ^"%s^"", g_aName[g_aNum], g_aPassword[g_aNum], szAccess, szFlags)
    write_file(filename, lineToAdd)
    ++g_aNum
  }

  log_to_file(g_logFile,_T("Loaded %d %s from database"),g_aNum,(g_aNum == 1) ? _T("admin") : _T("admins"))
  mysql_close(mysql)
  
  new ip[9]
  for(new id = 1; id <= g_maxplayers; id++) {
    if(is_user_authorized(id))
      if(get_cvar_num("amx_mode") > 0)
        accessUser(id)
    if(is_user_connected(id)) {
      if(!is_dedicated_server()) {
        ip[0] = '^0'
        get_user_ip(id, ip, 8, 1)
        if(equal(ip, "loopback"))
          accessUser(id)
      }
    }
  }
  return PLUGIN_HANDLED
}
#endif

loadSettings(szFilename[], canModify) {
  if(!file_exists(szFilename)) 
    return 0

  remove_user_flags(0, read_flags("z"))
  
  new szText[256], szFlags[32], szAccess[32], szComment[32]
  new a, pos = 0, modified = 0
  new parsedNum
  new comment[64]
  new szTest[32]

  g_aNum = 0

  while(g_aNum < MAX_ADMINS && (pos = read_file(szFilename, pos, szText, 255, a))) {
    if(!a || szText[0] == ';' || szText[0] == '/') continue

    parsedNum = parse(szText, g_aName[g_aNum], 31, g_aPassword[g_aNum], 31, szAccess, 31, szFlags, 31, szComment, 31)
    if(parsedNum < 4) continue

    comment[0] = '^0'
    if(parsedNum > 4) {
      new len = strlen(g_aName[g_aNum]) + strlen(g_aPassword[g_aNum]) + strlen(szAccess) + strlen(szFlags) + 4
      while(len < a && szText[len] != ';') {
        len++
      }
      if(len != a) {
        copy(comment, 63, szText[len+1])
      }
    }

    while(replace(szAccess, 31, " ", "")) { modified = 1; }
    while(replace(szFlags, 31, " ", "")) { modified = 1; }

    g_aAccess[g_aNum] = read_flags(szAccess)
    if((g_aAccess[g_aNum] & ADMIN_USER) != 0 && g_aAccess[g_aNum] != ADMIN_USER) {
      g_aAccess[g_aNum] -= ADMIN_USER
      while(replace(szAccess, 31, "z", "")) { modified = 1; }
    }

    if(canModify) {
      szTest[0] = '^0'
      parse(g_aName[g_aNum], szTest, 31)
      if(equali(szTest, "STEAM_", 6)) {
        if(containi(szFlags, "b") != -1) {
          while(replace(szFlags, 31, "b", "")) { modified = 1; }
        }
        if(containi(szFlags, "d") != -1) {
          while(replace(szFlags, 31, "d", "")) { modified = 1; }
        }
        if(containi(szFlags, "c") == -1) {
          add(szFlags, 31, "c")
          modified = 1
        }
      }
      else {
        if(containi(szFlags, "c") != -1) {
          while(replace(szFlags, 31, "c", "")) { modified = 1; }
        }
      }
      new i = 0, dots = 0
      while(isdigit(szTest[i]) || szTest[i] == '.')
        if(szTest[i++] == '.')
          dots++
      if(i == strlen(szTest) && dots == 3) {
        if(containi(szFlags, "b") != -1) {
          while(replace(szFlags, 31, "b", "")) { modified = 1; }
        }
        if(containi(szFlags, "c") != -1) {
          while(replace(szFlags, 31, "c", "")) { modified = 1; }
        }
        if(containi(szFlags, "d") == -1) {
          add(szFlags, 31, "d")
          modified = 1
        }
      }
      if(!dots && !equali(szTest, "loopback")) {
        if(containi(szFlags, "d") != -1) {
          while(replace(szFlags, 31, "d", "")) { modified = 1; }
        }
      }

      if(g_aPassword[g_aNum][0] == ' ') {
        new len = strlen(g_aPassword[g_aNum])
        if(len == 1) {
          g_aPassword[g_aNum][0] = '^0'
          modified = 1
        }
        else {
          new blank = 1
          for(i = 1; i < len; i++) {
            if(g_aPassword[g_aNum][i] == ' ') blank++
          }
          if(blank == len) {
            g_aPassword[g_aNum][0] = '^0'
            modified = 1
          }
        }
      }

      if(containi(szFlags, "c") != -1 || containi(szFlags, "d") != -1) {
        while(replace(g_aName[g_aNum], 31, " ", "")) { modified = 1; }
        while(replace(szFlags, 31, "a", "")) { modified = 1; }
        if(containi(szFlags, "e") == -1) {
          add(szFlags, 31, "e")
          modified = 1
        }
      }
      else {
        while(replace(szFlags, 31, "e", "")) { modified = 1; }
        if(containi(szFlags, "a") == -1) {
          add(szFlags, 31, "a")
          modified = 1
        }
        if(!g_aPassword[g_aNum][0]) {
          copy(g_aPassword[g_aNum][0], 31, "password")
        }
      }
    }
  
    g_aFlags[g_aNum] = read_flags(szFlags)

    if(comment[0]) {
      format(szText, 255, "^"%s^" ^"%s^" ^"%s^" ^"%s^" ;%s", g_aName[g_aNum], g_aPassword[g_aNum], szAccess, szFlags, comment)
    }
    else {
      format(szText, 255, "^"%s^" ^"%s^" ^"%s^" ^"%s^"", g_aName[g_aNum], g_aPassword[g_aNum], szAccess, szFlags)
    }
    if(canModify && modified) write_file(szFilename, szText, pos-1)

    ++g_aNum
  }

  log_to_file(g_logFile,_T("Loaded %d %s from %s"),g_aNum,(g_aNum == 1) ? _T("admin") : _T("admins"),szFilename)
  
  new ip[9]
  for(new id = 1; id <= g_maxplayers; id++) {
    if(is_user_authorized(id))
      if(get_cvar_num("amx_mode") > 0)
        accessUser(id)
    if(is_user_connected(id)) {
      if(!is_dedicated_server()) {
        ip[0] = '^0'
        get_user_ip(id, ip, 8, 1)
        if(equal(ip, "loopback")) {
          accessUser(id)
        }
      }
    }
  }
  return g_aNum
}

getAccess(id,name[],authid[],ip[],password[]) {
  new index = -1
  new result = 0
  for(new i = 0; i < g_aNum; ++i) {
    if(g_aFlags[i] & FLAG_AUTHID) {
      if(equal(authid, g_aName[i])) {
        index = i
        break
      }
    }
    else if(g_aFlags[i] & FLAG_IP) {
      new c = strlen(g_aName[i])
      if(g_aName[i][c-1] == '.') { /* check if this is not a xxx.xxx. format */
        if(equal(g_aName[i], ip, c)) {
          index = i
          break
        }
      } /* in other case an IP must just match */
      else if(equal(ip, g_aName[i])) {
        index = i
        break
      }
    }
    else {
      if(g_aFlags[i] & FLAG_TAG) {
        if(contain(name, g_aName[i]) != -1) {
          index = i
          break
        }
      }
      else if(equal(name, g_aName[i])) {
        index = i
        break
      }
    }
  }
  if(index != -1) {
    if(g_aFlags[index] & FLAG_NOPASS) {
      result |= 8
      new sflags[32]
      get_flags(g_aAccess[index], sflags, 31)
      set_user_flags(id, g_aAccess[index])
      log_to_file(g_logFile,"Login: ^"%s<%d><%s><>^" became an admin (account ^"%s^") (access ^"%s^") (address ^"%s^")",
        name,get_user_userid(id),authid,g_aName[index],sflags,ip)
    }
    else if(equal(password, g_aPassword[index])) {
      result |= 12
      set_user_flags(id, g_aAccess[index])
      new sflags[32]
      get_flags(g_aAccess[index], sflags, 31)
      log_to_file(g_logFile,"Login: ^"%s<%d><%s><>^" became an admin (account ^"%s^") (access ^"%s^") (address ^"%s^")",
        name,get_user_userid(id),authid,g_aName[index],sflags,ip)
    }
    else {
      result |= 1
      if(g_aFlags[index] & FLAG_KICK) {
        result |= 2
        log_to_file(g_logFile,"Login: ^"%s<%d><%s><>^" kicked due to invalid password (account ^"%s^") (address ^"%s^")",
          name,get_user_userid(id),authid,g_aName[index],ip)
      }
    }
  }
  else if(get_cvar_num("amx_mode") == 2) {
    result |= 2
  }
  else {
    new defaccess[32]
    get_cvar_string("amx_default_access", defaccess, 31)
    if(!strlen(defaccess))
      copy(defaccess, 31, "z")
    new idefaccess = read_flags(defaccess)
    if(idefaccess) {
      result |= 8
      set_user_flags(id, idefaccess)
    }
  }
  return result
}

accessUser(id,name[] = "") {
  remove_user_flags(id)
  new userip[32], userauthid[32], password[32], passfield[32], username[32]
  get_user_ip(id, userip, 31, 1)
  get_user_authid(id, userauthid, 31)
  if(name[0]) copy(username, 31, name)
  else get_user_name(id, username, 31)
  get_cvar_string("amx_password_field", passfield, 31)
  get_user_info(id, passfield, password, 31)
  new result = getAccess(id, username, userauthid, userip, password)
  if(result & 1) client_cmd(id, _T("echo ^"* Invalid Password!^""))
  if(result & 2) {
    client_cmd(id, g_cmdLoopback)
    return PLUGIN_HANDLED
  }
  if(result & 4) client_cmd(id, _T("echo ^"* Password accepted^""))
  if(result & 8) client_cmd(id, _T("echo ^"* Privileges set^""))
  return PLUGIN_CONTINUE
}

public client_infochanged(id) {
  if(!is_user_connected(id) || !get_cvar_num("amx_mode"))
    return PLUGIN_CONTINUE

  new newname[32], oldname[32]
  get_user_name(id, oldname, 31)
  get_user_info(id, "name", newname, 31)
  if(!equal(newname, oldname))
    accessUser(id, newname)
  return PLUGIN_CONTINUE
}

public ackSignal(id) {
  server_cmd("kick #%d %s", get_user_userid(id), _T("You have no entry to the server..."))
}

// So that the local player can get his access on listen servers
public client_connect(id) {
  g_whoGaveTempAdmin[id] = 0
  if(!is_dedicated_server()) {
    new ip[9]
    get_user_ip(id, ip, 8, 1)
    if(equal(ip, "loopback"))
      accessUser(id)
  }
}

public client_authorized(id) {
  return get_cvar_num("amx_mode") ? accessUser(id) : PLUGIN_CONTINUE
}

public cmdAddAdmin(id,level,cid) {
  if(!cmd_access(id,level,cid,5))
    return PLUGIN_HANDLED

  new szPassword[32]
  read_argv(2, szPassword, 31)
  new szAccess[32]
  read_argv(3, szAccess, 31)
  replace_all(szAccess, 31, " ", "")
  new szFlags[32]
  read_argv(4, szFlags, 31)
  replace_all(szFlags, 31, " ", "")

  new szName[32]
  read_argv(1, szName, 31)
  new player = cmd_target(id, szName, 8)
  if(player) {
    if(containi(szFlags, "c") != -1) get_user_authid(player, szName, 31)
    else if(containi(szFlags, "d") != -1) get_user_ip(player, szName, 31, 1)
    else get_user_name(player, szName, 31)
  }

#if !defined USE_SQL
  new filename[64]
  build_path(filename, 63, "$configdir/users.ini", filename)
  if(file_exists(filename)) {
    new szText[128], lineName[32], linePassword[32], lineAccess[32], lineFlags[32]
    new line = 0, len
    while((line = read_file(filename, line, szText, 127, len))) {
      if(!len || szText[0] == ';' || szText[0] == '/') continue

      if(parse(szText, lineName, 31, linePassword, 31, lineAccess, 31, lineFlags, 31) < 4) continue
      if(equali(szName, lineName)) {
        console_print(id, _T("[AMX] Can't add admin, %s is already admin"), szName)
        return PLUGIN_HANDLED
      }
    }
  }
#else
  new host[64], user[32], pass[32], db[32], error[128]
  get_cvar_string("amx_mysql_host", host, 63)
  get_cvar_string("amx_mysql_user", user, 31)
  get_cvar_string("amx_mysql_pass", pass, 31)
  get_cvar_string("amx_mysql_db", db, 31)

  new mysql = mysql_connect(host, user, pass, db, error, 127)
  if(mysql < 1) {
    log_to_file(g_logFile,_T("MySQL error: can't connect: '%s'"),error)
    return PLUGIN_HANDLED
  }

  if(mysql_query(mysql, "SELECT auth,password,access,flags FROM admins") < 1) {
    mysql_close(mysql)
    return PLUGIN_HANDLED
  }

  new lineName[32]
  while(mysql_nextrow(mysql) > 0) {
    mysql_getfield(mysql, 1, lineName, 31)
    if(equali(szName, lineName)) {
      console_print(id, _T("[AMX] Can't add admin, %s is already admin"), szName)
      mysql_close(mysql)
      return PLUGIN_HANDLED
    }
  }
#endif

  new myname[32], myauthid[32]
  get_user_name(id, myname, 31)
  get_user_authid(id, myauthid, 31)
  new lineToAdd[256]
  format(lineToAdd, 255, "^"%s^" ^"%s^" ^"%s^" ^"%s^"", szName, szPassword, szAccess, szFlags)

#if !defined USE_SQL
  if(!write_file(filename, lineToAdd)) {
    console_print(id, _T("[AMX] Failed writing to %s!"), filename)
    return PLUGIN_HANDLED
  }
  console_print(id, _T("[AMX] Added line %s to %s"), lineToAdd, filename)
  log_to_file(g_logFile,"AddAdmin: ^"%s<%d><%s><>^" add line %s to %s", 
    myname,get_user_userid(id),myauthid,lineToAdd,filename)
  loadSettings(filename, get_cvar_num("amx_correct_usersfile")) // Load admins accounts
#else
  new query[256]
  format(query, 255, "INSERT INTO admins (auth,password,access,flags) VALUES ('%s','%s','%s','%s')", szName, szPassword, szAccess, szFlags)
  if(mysql_query(mysql, query) < 1) {
    mysql_error(mysql, error, 127)
    console_print(id, _T("MySQL error: can't add admin: '%s'"), error)
    mysql_close(mysql)
    return PLUGIN_HANDLED
  }
  mysql_close(mysql)
  console_print(id, _T("[AMX] Added line %s to MySQL db"), lineToAdd)
  log_to_file(g_logFile,"AddAdmin: ^"%s<%d><%s><>^" add line %s to MySQL db.", 
    myname,get_user_userid(id),myauthid,lineToAdd)
  server_cmd("amx_loadsqladmins")
#endif
  return PLUGIN_HANDLED
}

public cmdRemoveAdmin(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new szName[32]
  read_argv(1, szName, 31)
  new szSteamid[32], szIP[32]
  szSteamid[0] = '^0'
  szIP[0] = '^0'
  new player = cmd_target(id, szName, 8)
  if(player) {
    get_user_name(player, szName, 31)
    get_user_authid(player, szSteamid, 31)
    get_user_ip(player, szIP, 31, 1)
  }
  new bool:bRemoved = false

#if !defined USE_SQL
  new filename[64]
  build_path(filename, 63, "$configdir/users.ini", filename)
  if(file_exists(filename)) {
    new szText[128], lineName[32], linePassword[32], lineAccess[32], lineFlags[32]
    new line = 0, len
    while((line = read_file(filename, line, szText, 127, len))) {
      if(!len || szText[0] == ';' || szText[0] == '/') continue

      if(parse(szText, lineName, 31, linePassword, 31, lineAccess, 31, lineFlags, 31) < 4) continue
      if(equali(szName, lineName) || (szSteamid[0] && equali(szSteamid, lineName)) || (szIP[0] && equali(szIP, lineName))) {
        write_file(filename, "", line-1)
        console_print(id, _T("[AMX] Removed admin ^"%s^""), lineName)
        bRemoved = true
      }
    }
  }
  if(bRemoved == true) {
    loadSettings(filename, get_cvar_num("amx_correct_usersfile")) // Load admins accounts
  }
#else
  new host[64], user[32], pass[32], db[32], error[128], query[256]
  get_cvar_string("amx_mysql_host", host, 63)
  get_cvar_string("amx_mysql_user", user, 31)
  get_cvar_string("amx_mysql_pass", pass, 31)
  get_cvar_string("amx_mysql_db", db, 31)

  new mysql = mysql_connect(host, user, pass, db, error, 127)
  if(mysql < 1) {
    log_to_file(g_logFile,_T("MySQL error: can't connect: '%s'"),error)
    return PLUGIN_HANDLED
  }

  if(mysql_query(mysql, "SELECT auth,password,access,flags FROM admins") < 1) {
    mysql_close(mysql)
    return PLUGIN_HANDLED
  }

  new lineName[32], len = 0
  while(mysql_nextrow(mysql) > 0) {
    mysql_getfield(mysql, 1, lineName, 31)
    if(equali(szName, lineName) || (szSteamid[0] && equali(szSteamid, lineName)) || (szIP[0] && equali(szIP, lineName))) {
      if(bRemoved == false) {
        len = format(query, 255, "DELETE FROM admins WHERE auth='%s'", lineName)
        bRemoved = true
      }
      else {
        len += format(query[len], 255-len, " OR auth='%s'", lineName)
      }

    }
  }
  if(bRemoved == true) {
    if(mysql_query(mysql, query) < 1) {
      mysql_error(mysql, error, 127)
      console_print(id, _T("MySQL error: can't remove admin: '%s'"), error)
      mysql_close(mysql)
      return PLUGIN_HANDLED
    }
    else {
      replace_all(query, 255, "DELETE FROM admins WHERE auth=", "")
      replace_all(query, 255, "OR auth=", "")
      console_print(id, _T("[AMX] Removed admin ^" %s ^""), query)
    }
  }
  mysql_close(mysql)
  if(bRemoved == true) {
    server_cmd("amx_loadsqladmins")
  }
#endif

  return PLUGIN_HANDLED
}

public cmdMakeAdmin(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new szPlayer[32]
  read_argv(1, szPlayer, 31)
  new player = cmd_target(id, szPlayer, 8)
  if(!player) return PLUGIN_HANDLED
  new name2[32]
  get_user_name(player, name2, 31)
  if(is_user_admin(player) && !g_whoGaveTempAdmin[player]) {
    console_print(id, _T("[AMX] %s is already admin"), name2)
    return PLUGIN_HANDLED
  }

  new szAccess[32]
  read_argv(2, szAccess, 31)
  new accessflags = read_flags(szAccess)
  if((accessflags & ADMIN_USER) != 0 && accessflags != ADMIN_USER) {
    accessflags -= ADMIN_USER
  }
  if(accessflags > 0) {
    if((accessflags & ADMIN_IMMUNITY) != 0) accessflags -= ADMIN_IMMUNITY
    if((accessflags & ADMIN_PASSWORD) != 0) accessflags -= ADMIN_PASSWORD
    if((accessflags & ADMIN_RCON) != 0) accessflags -= ADMIN_RCON
  }
  if(accessflags < 0) {
    console_print(id, _T("[AMX] ^"%s^" are not correct access flags"), szAccess)
    return PLUGIN_HANDLED
  }

  new name[32]
  get_user_name(id, name, 31)
  remove_user_flags(player)
  if(accessflags == 0) {
    get_cvar_string("amx_default_access", szAccess, 31)
    if(!strlen(szAccess))
      copy(szAccess, 31, "z")
    accessflags = read_flags(szAccess)
    g_whoGaveTempAdmin[player] = 0

    switch(get_cvar_num("amx_show_activity")) {
      case 2: client_print(0, print_chat, _T("ADMIN %s: give back default access flags to %s"), name, name2)
      case 1: client_print(0, print_chat, _T("ADMIN: give back default access flags to %s"), name2)
    }

    new authid[32], authid2[32]
    get_user_authid(id, authid, 31)
    get_user_authid(player, authid2, 31)
    log_to_file(g_logFile,"MakeAdmin: ^"%s<%d><%s><>^" give back default access flags to ^"%s<%d><%s><>^"", 
      name,get_user_userid(id),authid,name2,get_user_userid(player),authid2)

    console_print(id, _T("[AMX] ^"%s^" get back the default access flags"), name2)
    client_print(player, print_chat, _T("[AMX] %s gave you back the default access flags"), name)
  }
  else {
    g_whoGaveTempAdmin[player] = id

    switch(get_cvar_num("amx_show_activity")) {
      case 2: client_print(0, print_chat, _T("ADMIN %s: give admin powers to %s"), name, name2)
      case 1: client_print(0, print_chat, _T("ADMIN: give admin powers to %s"), name2)
    }

    get_flags(accessflags, szAccess, 31)
    new authid[32], authid2[32]
    get_user_authid(id, authid, 31)
    get_user_authid(player, authid2, 31)
    log_to_file(g_logFile,"MakeAdmin: ^"%s<%d><%s><>^" give admin powers to ^"%s<%d><%s><>^" (flags ^"%s^")", 
      name,get_user_userid(id),authid,name2,get_user_userid(player),authid2,szAccess)

    console_print(id, _T("[AMX] ^"%s^" is now admin with access flags ^"%s^""), name2, szAccess)
    client_print(player, print_chat, _T("[AMX] %s gave you admin powers (flags ^"%s^")"), name, szAccess)
  }
  set_user_flags(player, accessflags)
  return PLUGIN_HANDLED
}

public client_disconnect(id) {
  new szAccess[32], name[32]
  for(new i = 1; i <= g_maxplayers; i++) {
    if(!is_user_connected(i)) continue
    if(g_whoGaveTempAdmin[i] == id) {
      szAccess[0] = '^0'
      get_cvar_string("amx_default_access", szAccess, 31)
      if(!strlen(szAccess))
        copy(szAccess, 31, "z")
      remove_user_flags(i)
      set_user_flags(i, read_flags(szAccess))
      g_whoGaveTempAdmin[i] = 0
      name[0] = '^0'
      get_user_name(id, name, 31)
      client_print(i, print_chat, _T("[AMX] %s changed your access, but left. Returning you to your original access."), name)
    }
  }
}
