/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/* 
 * (c) 2003, tcquest78
 *
 * Provides help for admin commands with amx_help command
 *
 */

#include <translator>
#include <amxmod>

// Comment out if you don't want AMX to show the help message after joining the server
#define SHOW_HELPMSG

#define HELPAMOUNT 10 // Number of commands per page

new g_typeHelp[] = "Type 'amx_help' in the console to see available commands."
new g_typeHelpMod[] = "Type 'amx_modules' in the console to see running modules."
new g_timeInfo1[] = "Time Left: %d:%02d min. Next Map: %s."
new g_timeInfo2[] = "No Time Limit. Next Map: %s."

public plugin_init() {
  load_translations("adminhelp")
  register_plugin(_T("Admin Help"),AMX_VERSION,"AMX Team")
  register_concmd("amx_help","cmdHelp",0,_T("- displays this help"))
}

#if defined SHOW_HELPMSG
public client_putinserver(id) {
  if(!is_user_bot(id)) setHelp(id)
}

public client_disconnect(id) {
  remove_task(134679+id)
}
#endif

public cmdHelp(id,level,cid) {
  new arg1[8], flags = get_user_flags(id)
  new start = read_argv(1, arg1, 7) ? str_to_num(arg1) : 1
  if(--start < 0) start = 0
  new clcmdsnum = get_concmdsnum(flags, id)
  if(start >= clcmdsnum) start = clcmdsnum - 1
  console_print(id, _T("^n----- AMX Help: Commands -----"))
  new info[128], cmd[32], eflags
  new end = start + HELPAMOUNT
  if(end > clcmdsnum) end = clcmdsnum
  for(new i = start; i < end; i++) {
    get_concmd(i, cmd, 31, eflags, info, 127, flags, id)
    info[120-strlen(cmd)] = '^0'
    console_print(id, "%3d: %s %s", i+1, cmd, info)
  }
  console_print(id, _T("----- Entries %d - %d of %d -----"), start+1, end, clcmdsnum)
  if(end < clcmdsnum)
     console_print(id, _T("----- Use 'amx_help %d' for more -----"), end+1)
  else
     console_print(id, _T("----- Use 'amx_help 1' for begin -----"))
  return PLUGIN_HANDLED
}

public dispInfo(id) {
  id -= 134679
  if(is_user_connected(id)) {
    client_print(id, print_chat, _T(g_typeHelp))
    client_print(id, print_chat, _T(g_typeHelpMod))
    new nextmap[32]
    new winlimit = get_cvar_num("mp_winlimit")
    new maxrounds = get_cvar_num("mp_maxrounds")
    if(xvar_exists("NEXTMAP_MSG") && get_xvar_num(get_xvar_id("NEXTMAP_MSG")) != 3 && (get_timeleft() != 0 || winlimit || maxrounds)) {
      copy(nextmap, 31, _T("selected by vote",id))
    }
    else {
      get_cvar_string("amx_nextmap", nextmap, 31)
    }
    if(get_cvar_float("mp_timelimit") > 0.0) {
      new timeleft = get_timeleft()
      if(timeleft > 0) {
        client_print(id, print_chat, _T(g_timeInfo1), timeleft / 60, timeleft % 60, nextmap)
      }
    }
    else {
      client_print(id, print_chat, _T(g_timeInfo2), nextmap)
    }
  }
}

#if defined SHOW_HELPMSG
setHelp(id) {
  set_task(15.0, "dispInfo", 134679+id)
}
#endif
