/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * amx_reservation = 1 or 2 or 3
 *   Set your server max_players to 1 above the desired value (21 instead of 20 for instance).
 *   Query programs will see max_players-1 as max. Admins with reservation
 *   can connect when there is 20/20 and player with worst ping or shortest playing time
 *   or longest playing time will be kicked.
 *
 * Cvars:
 *
 * amx_reservation <value>
 * 0 - Disables the plugin.
 * 1 - Kicks the Player with the shortest playing time when an admin connects to a full server.
 * 2 - Kicks the Player with the longest playing time when an admin connects to a full server.
 * 3 - Kicks the Player with the highest ping when an admin connects to a full server.
 * 4 - AMX 099 Mode: 'amx_reservedslots' are reserved, nobody is kicked when an admin connects to a full server
 *
 * amx_reservedslots <value> : number of reserved slots (if amx_reservation is set to 4)
 *
 * amx_hideslots <1 or 0> : hide reserved slots or not (if amx_reservation is set to 4)
 *
 *
 * The code is from f117bomb's admin slots plugin (with some modifications).
 *
 */

#include <translator>
#include <amxmod>
#include <amxmisc>

new g_cmdLoopback[16]
new g_maxplayers

public plugin_init() {
  load_translations("adminslots")
  register_plugin(_T("Slots Reservation"),AMX_VERSION,"AMX Team")
  register_cvar("amx_reservation","0")
  register_cvar("amx_reservedslots","0")
  register_cvar("amx_hideslots","0")

  format(g_cmdLoopback, 15, "amxres%c%c%c%c" ,
    random_num('A','Z'), random_num('A','Z'), random_num('A','Z'), random_num('A','Z'))
  register_clcmd(g_cmdLoopback, "ackSignal")

  g_maxplayers = get_maxplayers()
}

public plugin_cfg() {
  new amx_reservation = get_cvar_num("amx_reservation")
  new amx_hideslots = get_cvar_num("amx_hideslots")
  new amx_reservedslots = get_cvar_num("amx_reservedslots")
  if(amx_reservation > 0 && amx_reservation < 4) {
    set_cvar_num("sv_visiblemaxplayers", g_maxplayers - 1)
  }
  else if(amx_reservation == 4) {
    if(amx_hideslots == 1) {
      new players = get_playersnum(1)
      new limit = g_maxplayers - amx_reservedslots
      setVisibleSlots(players, limit)
    }
  }
}

public ackSignal(id) {
  server_cmd("kick #%d %s", get_user_userid(id), _T("Dropped due to slot reservation",id))
}

public client_authorized(id) {
  new players = get_playersnum(1)
  new limit
  new amx_reservation = get_cvar_num("amx_reservation")
  if(amx_reservation <= 0) return PLUGIN_CONTINUE

  if(amx_reservation < 4) {
    new who
    limit = g_maxplayers - 1
    if(players > limit) {
      if(get_user_flags(id) & ADMIN_RESERVATION) {
        switch(amx_reservation) {
          case 1:
            who = kickShortestPlayingTime()
          case 2:
            who = kickLongestPlayingTime()
          case 3:
            who = kickLag()
        }
        if(who) {
          new name[32]
          get_user_name(who, name, 31)
          client_cmd(id, _T("echo ^"* %s was kicked to free this slot^""), name)
        }
        return PLUGIN_CONTINUE
      }
      client_cmd(id, "%s", g_cmdLoopback)
      return PLUGIN_HANDLED
    }
  }
  else if(amx_reservation == 4) {
    limit = g_maxplayers - get_cvar_num("amx_reservedslots")
    if((get_user_flags(id) & ADMIN_RESERVATION) || (players <= limit)) {
      if(get_cvar_num("amx_hideslots") == 1) {
        setVisibleSlots(players, limit)
      }
      return PLUGIN_CONTINUE
    }
    client_cmd(id, "%s", g_cmdLoopback)
  }
  return PLUGIN_CONTINUE
}

public client_disconnect(id) {
  if(get_cvar_num("amx_reservation") == 4 && get_cvar_num("amx_hideslots") == 1) {
    setVisibleSlots(get_playersnum(1) - 1, g_maxplayers - get_cvar_num("amx_reservedslots"))
  }
  return
}

kickShortestPlayingTime() {
  new who = 0, itime, shortest = 0x7fffffff
  for(new i = 1; i <= g_maxplayers; ++i) {
    if(!is_user_connected(i) && !is_user_connecting(i))
      continue
    if(get_user_flags(i) & ADMIN_RESERVATION)
      continue
    itime = get_user_time(i)
    if(shortest > itime) {
      shortest = itime
      who = i
    }
  }
  if(who) {
    client_cmd(who, _T("echo ^"Dropped due to shortest playing time to free slot for an admin^";%s"), g_cmdLoopback)
  }
  return who
}

kickLongestPlayingTime() {
  new who = 0, itime, longest = -1
  for(new i = 1; i <= g_maxplayers; ++i) {
    if(!is_user_connected(i) && !is_user_connecting(i))
      continue
    if(get_user_flags(i) & ADMIN_RESERVATION)
      continue
    itime = get_user_time(i)
    if(longest < itime) {
      longest = itime
      who = i
    }
  }
  if(who) {
    client_cmd(who, _T("echo ^"Dropped due to longest playing time to free slot for an admin^";%s"), g_cmdLoopback)
  }
  return who
}

kickLag() {
  new who = 0, ping, loss, worst = -1
  for(new i = 1; i <= g_maxplayers; ++i) {
    if(!is_user_connected(i) && !is_user_connecting(i))
      continue
    if(get_user_flags(i) & ADMIN_RESERVATION)
      continue
    get_user_ping(i, ping, loss)
    if(ping > worst) {
      worst = ping
      who = i
    }
  }
  if(who) {
    client_cmd(who, _T("echo ^"Dropped due to high ping to free slot for an admin^";%s"), g_cmdLoopback)
  }
  return who
}

setVisibleSlots(players, limit) {
  new num = players + 1
  if(players == g_maxplayers) {
    num = g_maxplayers
  }
  else if(players < limit) {
    num = limit
  }
  set_cvar_num("sv_visiblemaxplayers", num)
}
