/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/* 
 *
 * I. Vote for changing the level:
 * amx_votemap < map > [ map ] [ map ] [ map ]
 *
 * Examples:
 * amx_votemap "de_dust"
 * amx_votemap "de_dust" "de_dust2" "cs_italy"
 *
 * II. Vote for kicking the player:
 * amx_votekick <name|#userid|authid>
 *
 * Examples:
 * amx_votekick "Player"
 *
 * III. Vote for banning and kicking the player for 30 minutes:
 * amx_voteban <name|#userid|authid>
 *
 * Examples:
 * amx_voteban "Player"
 *
 * IV. Custom voting:
 * amx_vote < question > < g_Answer1 > < g_Answer2 > [ g_Answer3 ] [ g_Answer4 ]
 *
 * Examples:
 * amx_vote "sv_restart" "5" "0" 
 * // this will change the value of the cvar if you accept the result
 * // (You need the "g" admin flag)
 *
 * amx_vote "mp_timelimit" "20" "30" "40" "50"
 * // this will change the value of the cvar if you accept the result
 * // (You need the "g" admin flag)
 *
 * amx_vote "csdm_enable" "1" "0"
 * // this will enable/disable CSDM if you accept the result
 * // (You need the admin flag required by csdm_enable)
 *
 * amx_vote "are you hungry?" "yes" "no" "maybe"
 *
 */

#include <translator>
#include <amxmod>
#include <amxmisc>

#define CHECK_MENU_TIME 0.5

new g_Answer[128]
new g_optionName[4][32]
new g_voteCount[4]
new g_validMaps
new g_yesNoVote
new g_logFile[16]
new g_coloredMenus
new g_voteCaller
new bool:g_hasVoted[33]
new g_Execute[256]
new g_execLen
new g_menukeys
new g_menumsg[33][256]
new g_menutitle[16]

new g_alreadyVoting[]= "There is already one voting..."
new g_notAllowed[] = "Voting not allowed at this time"
new g_votingStarted[] = "Voting has started..."
new g_resultRef[] = "Result refused"
new g_resultAcpt[] = "Result accepted"
new g_votingFailed[] = "Voting failed"
new g_votingSuccess[] = "Voting successful"

new g_playerTag[] = "PLAYER"
new g_adminTag[] = "ADMIN"

new bool:g_execResult
new Float:g_voteRatio
new g_maxplayers

new menuid_changemap, menuid_choosemap, menuid_kick, menuid_ban, menuid_vote

public plugin_init() {
  load_translations("adminvote")
  register_plugin(_T("Admin Votes"),AMX_VERSION,"AMX Team")

  menuid_changemap = register_menuid("Change map to ")
  menuid_choosemap = register_menuid("Choose map: ")
  menuid_kick = register_menuid("Kick ")
  menuid_ban = register_menuid("Ban ")
  menuid_vote = register_menuid("Vote: ")
  register_menucmd(menuid_changemap,(1<<0)|(1<<1),"voteCount")
  register_menucmd(menuid_choosemap,(1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<9),"voteCount")
  register_menucmd(menuid_kick,(1<<0)|(1<<1),"voteCount")
  register_menucmd(menuid_ban,(1<<0)|(1<<1),"voteCount")
  register_menucmd(menuid_vote,(1<<0)|(1<<1)|(1<<2)|(1<<3),"voteCount")
  register_menucmd(register_menuid("The result: "),(1<<0)|(1<<1),"actionResult")

  register_concmd("amx_votemap","cmdVoteMap",ADMIN_VOTE,_T("<map> [map] [map] [map]"))
  register_concmd("amx_votekick","cmdVoteKickBan",ADMIN_VOTE,_T("<name|#userid|authid>"))
  register_concmd("amx_voteban","cmdVoteKickBan",ADMIN_VOTE,_T("<name|#userid|authid> [ip]"))
  register_concmd("amx_vote","cmdVote",ADMIN_VOTE,_T("<question> <answer#1> <answer#2> [answer#3] [answer#4]"))
  register_concmd("amx_cancelvote","cmdCancelVote",ADMIN_VOTE,_T("- cancels last vote"))

  g_coloredMenus = colored_menus()
  g_maxplayers = get_maxplayers()
  get_logfile(g_logFile, 15)
}

canChangeCvar(id, const cvar[]) {
  if(equali(cvar, "amx_version")
  || equali(cvar, "fun_version")
  || equali(cvar, "amx_last_voting"))
    return 0
  if(!(get_user_flags(id) & ADMIN_RCON) &&
  (  equali(cvar, "sv_cheats")
  || equali(cvar, "sv_password")
  || equali(cvar, "rcon_password")
  || equali(cvar, "amx_show_activity")
  || equali(cvar, "amx_mode")
  || equali(cvar, "amx_password_field")
  || equali(cvar, "amx_default_access")
  || equali(cvar, "amx_reservation")
  || equali(cvar, "amx_debug")
  || equali(cvar, "amx_langdebug")
  || equali(cvar, "csstats_rank")
  || equali(cvar, "csstats_rankbots")
  || equali(cvar, "csstats_maxsize")
  || equali(cvar, "csstats_reset")
  || equali(cvar, "dodstats_rank")
  || equali(cvar, "dodstats_rankbots")
  || equali(cvar, "dodstats_pause")
  || equali(cvar, "dodstats_maxsize")
  || equali(cvar, "dodstats_reset")))
    return 0
  return 1
}

public check_menu(param[]) {
  new iter = param[0]
  new Float:vote_time = float(param[2]) + 2.0 - (iter * CHECK_MENU_TIME)
  new menutime = floatround(vote_time, floatround_floor)
  if(menutime > 0) {
    new players[32], numplayers
    new menuid, keys, tempId
    get_players(players, numplayers, "c")
    for(numplayers--; numplayers >= 0; numplayers--) {
      tempId = players[numplayers]
      if(g_hasVoted[tempId]) continue
      get_user_menu(tempId, menuid, keys)
      if(keys == 0 || menuid < 0) {
        show_menu(tempId, g_menukeys, g_menumsg[tempId], menutime, g_menutitle)
      }
      else {
        if(menuid != param[1]) {
          if(menutime <= 6) {
            show_menu(tempId, g_menukeys, g_menumsg[tempId], menutime, g_menutitle)
          }
        }
        else if(tempId == g_voteCaller) {
          show_menu(tempId, g_menukeys, g_menumsg[tempId], menutime, g_menutitle)
        }
      }
    }
    ++param[0]
    set_task(CHECK_MENU_TIME, "check_menu", 1467851, param, 3)
  }
}

public cmdCancelVote(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  if(task_exists(99889988, 1)) {
    if(is_user_connected(id)) {
      new authid[32], name[32]
      get_user_authid(id, authid, 31)
      get_user_name(id, name, 31)

      log_to_file(g_logFile,"Vote: ^"%s<%d><%s><>^" cancel vote session",name,get_user_userid(id),authid)
      switch(get_cvar_num("amx_show_activity")) {
        case 2: {
          if(is_user_admin(id)) {
            client_print(0, print_chat, _T("%s %s: cancel vote"), _T(g_adminTag), name)
          }
          else {
            client_print(0, print_chat, _T("%s %s: cancel vote"), _T(g_playerTag), name)
          }
        }
        case 1: {
          if(is_user_admin(id)) {
            client_print(0, print_chat, _T("%s: cancel vote"), _T(g_adminTag))
          }
          else {
            client_print(0, print_chat, _T("%s: cancel vote"), _T(g_playerTag))
          }
        }
      }
    }

    console_print(id, _T("Voting canceled"))
    client_print(0, print_chat, _T("Voting canceled"))
    remove_task(99889988, 1)
    set_cvar_float("amx_last_voting", get_gametime())

    remove_task(1467851)
    new menuid, keys
    for(new i = 1; i <= g_maxplayers; i++) {
      g_hasVoted[i] = true
      if(!is_user_connected(i)) continue
      get_user_menu(i, menuid, keys)
      if(menuid == menuid_changemap
      || menuid == menuid_choosemap
      || menuid == menuid_kick
      || menuid == menuid_ban
      || menuid == menuid_vote) {
        show_menu(i, 0, "NULL", 0, "NULL")
      }
    }
  }
  else {
    console_print(id, _T("There is no voting to cancel or the vote session can't be canceled with that command"))
  }
  return PLUGIN_HANDLED
}

public delayedExec(cmd[]) {
  server_cmd("%s", cmd)
}

public autoRefuse() {
  log_to_file(g_logFile,"Vote: %s",g_resultRef)
  client_print(0, print_chat, _T(g_resultRef))
}

public actionResult(id,key) {
  remove_task(4545454)

  switch(key) {
    case 0: {
      set_task(2.0, "delayedExec", 55119977, g_Execute, g_execLen)
      log_to_file(g_logFile,"Vote: %s",g_resultAcpt) 
      client_print(0, print_chat, _T(g_resultAcpt))
    }
    case 1:
      autoRefuse()
  }
  return PLUGIN_HANDLED
}

public checkVotes() {
  remove_task(1467851)
  for(new i = 1; i <= g_maxplayers; i++) {
    g_hasVoted[i] = true
  }

  new best = 0
  if(!g_yesNoVote) {
    for(new a = 0; a < 4; ++a)
      if(g_voteCount[a] > g_voteCount[best])
        best = a
  }

  new votesNum = g_voteCount[0] + g_voteCount[1] + g_voteCount[2] + g_voteCount[3]
  new iRatio = votesNum ? floatround(g_voteRatio * float(votesNum), floatround_ceil) : 1
  new iResult = g_voteCount[best]

  if(iResult < iRatio) {
    if(g_yesNoVote)
      client_print(0, print_chat, _T("%s (yes ^"%d^") (no ^"%d^") (needed ^"%d^")"), _T(g_votingFailed), g_voteCount[0], g_voteCount[1], iRatio)
    else
      client_print(0, print_chat, _T("%s (got ^"%d^") (needed ^"%d^")"), _T(g_votingFailed), iResult, iRatio)
    log_to_file(g_logFile,"Vote: %s (got ^"%d^") (needed ^"%d^")",g_votingFailed,iResult,iRatio)
    return PLUGIN_CONTINUE
  }

  g_execLen = format(g_Execute, 255, g_Answer, g_optionName[best]) + 1

  if(g_execResult) {
    g_execResult = false
    if(is_user_connected(g_voteCaller)) {
      new menuBody[512]
      new len
      if(g_coloredMenus) {
        len = format(menuBody, 511, _T("\yThe result: \w%s^n^n",g_voteCaller), g_Execute)
      }
      else {
        len = format(menuBody, 511, _T("The result: %s^n^n",g_voteCaller), g_Execute)
      }
      if(g_coloredMenus) {
        len += copy(menuBody[len], 511 - len, _T("\yDo you want to continue?^n\w",g_voteCaller))
      }
      else {
        len += copy(menuBody[len], 511 - len, _T("Do you want to continue?^n",g_voteCaller))
      }
      copy(menuBody[len], 511 - len, _T("^n1. Yes^n2. No",g_voteCaller))
      show_menu(g_voteCaller, 0x03, menuBody, 10, "The result: ")
      set_task(10.0, "autoRefuse", 4545454)
    }
    else {
      if(equal(g_Execute, "changelevel", 11)) {
        new modName[8]
        get_modname(modName, 7)
        if(!equal(modName, "zp")) {
          message_begin(MSG_ALL, SVC_INTERMISSION)
          message_end()
        }
      }
      set_task(1.0, "delayedExec", 55119966, g_Execute, g_execLen)
    }
  }

  client_print(0, print_chat, _T("%s (got ^"%d^") (needed ^"%d^"). The result: %s"), _T(g_votingSuccess), iResult, iRatio, g_Execute)
  log_to_file(g_logFile,"Vote: %s (got ^"%d^") (needed ^"%d^") (result ^"%s^")",g_votingSuccess,iResult,iRatio,g_Execute)
  return PLUGIN_CONTINUE
}

public voteCount(id,key) {
  if(g_hasVoted[id]) {
    return PLUGIN_HANDLED
  }
  g_hasVoted[id] = true
  if(get_cvar_num("amx_vote_answers")) {
    new name[32]
    get_user_name(id, name, 31)
    if(g_yesNoVote)
      client_print(0, print_chat, _T("%s voted %s"), name, key ? _T("against") : _T("for"))
    else
      client_print(0, print_chat, _T("%s voted for %s"), name, g_optionName[key])
  }
  ++g_voteCount[key]
  return PLUGIN_HANDLED
}

public cmdVoteMap(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new Float:voting = get_cvar_float("amx_last_voting")
  if(voting > get_gametime()) {
    console_print(id, _T(g_alreadyVoting))
    return PLUGIN_HANDLED
  }
  if(voting && voting + get_cvar_float("amx_vote_delay") > get_gametime()) {
    console_print(id, _T(g_notAllowed))
    return PLUGIN_HANDLED
  }

  new argc = read_argc()
  //if(argc > 5)
    //argc = 5

  g_validMaps = 0
  g_optionName[0][0] = '^0'
  g_optionName[1][0] = '^0'
  g_optionName[2][0] = '^0'
  g_optionName[3][0] = '^0'

  for(new i = 1; i < argc && g_validMaps < 4; ++i) {
    read_argv(i, g_optionName[g_validMaps], 31)
    if(is_map_valid(g_optionName[g_validMaps])) {
      replace_all(g_optionName[g_validMaps], 31, "<", "")
      replace_all(g_optionName[g_validMaps], 31, ">", "")
      g_validMaps++
    }
  }
  if(g_validMaps == 0) {
    console_print(id, (argc == 2) ? _T("Given map is not valid") : _T("Given maps are not valid"))
    return PLUGIN_HANDLED
  }

  for(new i = 1; i <= g_maxplayers; i++) {
    g_hasVoted[i] = false
  }

  g_voteCaller = id
  
  new menu_msg[256]
  new keys = 0
  new players[32]
  new numplayers
  new Float:vote_time = get_cvar_float("amx_vote_time") + 2.0

  get_players(players, numplayers, "c")
  if(g_validMaps > 1) {
    new temp[128]
    new menuid, tempkeys
    for(numplayers--; numplayers >= 0; numplayers--) {
      keys = (1<<9)
      if(g_coloredMenus) {
        copy(menu_msg, 255, _T("\yChoose map: \w^n^n",players[numplayers]))
      }
      else {
        copy(menu_msg, 255, _T("Choose map: ^n^n",players[numplayers]))
      }
      temp[0] = '^0'
      for(new a = 0; a < g_validMaps; ++a) {
        format(temp, 127, "%d.  %s^n", a+1, g_optionName[a])
        add(menu_msg, 255, temp)
        keys |= (1<<a)
      }
      add(menu_msg, 255, _T("^n0.  None",players[numplayers]))
      get_user_menu(players[numplayers], menuid, tempkeys)
      if(tempkeys == 0 || menuid < 0) {
        show_menu(players[numplayers], keys, menu_msg, floatround(vote_time), "Choose map: ")
      }
      g_menukeys = keys
      g_menumsg[players[numplayers]][0] = '^0'
      copy(g_menumsg[players[numplayers]], 255, menu_msg)
    }
    g_yesNoVote = 0
    g_menutitle[0] = '^0'
    copy(g_menutitle, 15, "Choose map: ")
    remove_task(1467851)
    new param[3]
    param[0] = 1
    param[1] = menuid_choosemap
    param[2] = get_cvar_num("amx_vote_time")
    set_task(CHECK_MENU_TIME, "check_menu", 1467851, param, 3)
  }
  else {
    new menuid, tempkeys
    for(numplayers--; numplayers >= 0; numplayers--) {
      if(g_coloredMenus) {
        format(menu_msg, 255, _T("\yChange map to %s?\w^n^n1.  Yes^n2.  No",players[numplayers]), g_optionName[0])
      }
      else {
        format(menu_msg, 255, _T("Change map to %s?^n^n1.  Yes^n2.  No",players[numplayers]), g_optionName[0])
      }
      keys = (1<<0)|(1<<1)
      get_user_menu(players[numplayers], menuid, tempkeys)
      if(tempkeys == 0 || menuid < 0) {
        show_menu(players[numplayers], keys, menu_msg, floatround(vote_time), "Change map to ")
      }
      g_menukeys = keys
      g_menumsg[players[numplayers]][0] = '^0'
      copy(g_menumsg[players[numplayers]], 255, menu_msg)
    }
    g_yesNoVote = 1
    g_menutitle[0] = '^0'
    copy(g_menutitle, 15, "Change map to ")
    remove_task(1467851)
    new param[3]
    param[0] = 1
    param[1] = menuid_changemap
    param[2] = get_cvar_num("amx_vote_time")
    set_task(CHECK_MENU_TIME, "check_menu", 1467851, param, 3)
  }

  if(is_user_connected(id)) {
    new authid[32], name[32]
    get_user_authid(id, authid, 31)
    get_user_name(id, name, 31)

    if(argc == 2)
      log_to_file(g_logFile,"Vote: ^"%s<%d><%s><>^" vote map (map ^"%s^")",name,get_user_userid(id),authid,g_optionName[0])
    else
      log_to_file(g_logFile,"Vote: ^"%s<%d><%s><>^" vote maps (map#1 ^"%s^") (map#2 ^"%s^") (map#3 ^"%s^") (map#4 ^"%s^")",name,get_user_userid(id),authid,g_optionName[0],g_optionName[1],g_optionName[2],g_optionName[3])

    switch(get_cvar_num("amx_show_activity")) {
      case 2: {
        if(is_user_admin(id)) {
          client_print(0, print_chat, _T("%s %s: vote map(s)"), _T(g_adminTag), name)
        }
        else {
          client_print(0, print_chat, _T("%s %s: vote map(s)"), _T(g_playerTag), name)
        }
      }
      case 1: {
        if(is_user_admin(id)) {
          client_print(0, print_chat, _T("%s: vote map(s)"), _T(g_adminTag))
        }
        else {
          client_print(0, print_chat, _T("%s: vote map(s)"), _T(g_playerTag))
        }
      }
    }
  }

  g_execResult = true
  set_cvar_float("amx_last_voting", get_gametime() + vote_time)
  g_voteRatio = get_cvar_float("amx_votemap_ratio")
  g_Answer = "changelevel %s"
  set_task(vote_time, "checkVotes", 99889988)
  console_print(id, _T(g_votingStarted))
  g_voteCount = {0, 0, 0, 0}
  return PLUGIN_HANDLED
}

public cmdVote(id,level,cid) {
  if(!cmd_access(id,level,cid,4))
    return PLUGIN_HANDLED

  new Float:voting = get_cvar_float("amx_last_voting")
  if(voting > get_gametime()) {
    console_print(id, _T(g_alreadyVoting))
    return PLUGIN_HANDLED
  }
  if(voting && voting + get_cvar_float("amx_vote_delay") > get_gametime()) {
    console_print(id, _T(g_notAllowed))
    return PLUGIN_HANDLED
  }

  new quest[48]
  read_argv(1, quest, 47)
  if((contain(quest, "sv_password") != -1) || (contain(quest, "rcon_password") != -1) || 
    (contain(quest, "kick") != -1) || (contain(quest, "addip") != -1) || (contain(quest, "ban") != -1)) {
    console_print(id, _T("Voting for that has been forbidden"))
    return PLUGIN_HANDLED
  }

  g_optionName[0][0] = 0
  g_optionName[1][0] = 0
  g_optionName[2][0] = 0
  g_optionName[3][0] = 0

  read_argv(2, g_optionName[0], 31)
  read_argv(3, g_optionName[1], 31)
  read_argv(4, g_optionName[2], 31)
  read_argv(5, g_optionName[3], 31)

  if(is_user_connected(id)) {
    new authid[32], name[32]
    get_user_authid(id, authid, 31)
    get_user_name(id, name, 31)

    log_to_file(g_logFile,"Vote: ^"%s<%d><%s><>^" vote custom (question ^"%s^") (option#1 ^"%s^") (option#2 ^"%s^") (option#3 ^"%s^") (option#4 ^"%s^")",name,get_user_userid(id),authid,quest,g_optionName[0],g_optionName[1],g_optionName[2],g_optionName[3])

    switch(get_cvar_num("amx_show_activity")) {
      case 2: {
        if(is_user_admin(id)) {
          client_print(0, print_chat, _T("%s %s: vote custom"), _T(g_adminTag), name)
        }
        else {
          client_print(0, print_chat, _T("%s %s: vote custom"), _T(g_playerTag), name)
        }
      }
      case 1: {
        if(is_user_admin(id)) {
          client_print(0, print_chat, _T("%s: vote custom"), _T(g_adminTag))
        }
        else {
          client_print(0, print_chat, _T("%s: vote custom"), _T(g_playerTag))
        }
      }
    }
  }

  for(new i = 1; i <= g_maxplayers; i++) {
    g_hasVoted[i] = false
  }

  g_voteCaller = id
  
  new menu_msg[256]
  new keys = (1<<0)|(1<<1)
  new players[32]
  new numplayers
  new Float:vote_time = get_cvar_float("amx_vote_time") + 2.0

  new cmdaccess[32], cmdflags, playerflags, command = 0
  if(get_cmdaccess(quest, cmdaccess, 31)) {
    cmdflags = read_flags(cmdaccess)
    playerflags = get_user_flags(id)
    command = 1
  }
  new yesNoVote = (command && read_argc() == 4 && ((g_optionName[0][0] == '0' && g_optionName[1][0] == '1') || (g_optionName[0][0] == '1' && g_optionName[1][0] == '0')))

  new len
  new menuid, tempkeys
  get_players(players, numplayers, "c")
  for(numplayers--; numplayers >= 0; numplayers--) {
    if(yesNoVote) {
      if(g_coloredMenus) {
        len = format(menu_msg, 255, _T("\yVote: %s\w^n^n1.  %s^n2.  %s",players[numplayers]), quest,
                   (g_optionName[0][0] == '0') ? _T("No",players[numplayers]) : _T("Yes",players[numplayers]),
                   (g_optionName[1][0] == '0') ? _T("No",players[numplayers]) : _T("Yes",players[numplayers]))
      }
      else {
        len = format(menu_msg, 255, _T("Vote: %s^n^n1.  %s^n2.  %s",players[numplayers]), quest,
                   (g_optionName[0][0] == '0') ? _T("No",players[numplayers]) : _T("Yes",players[numplayers]),
                   (g_optionName[1][0] == '0') ? _T("No",players[numplayers]) : _T("Yes",players[numplayers]))
      }
      g_menumsg[players[numplayers]][0] = '^0'
      copy(g_menumsg[players[numplayers]], 255, menu_msg)
    }
    else {
      if(g_coloredMenus) {
        len = format(menu_msg, 255, _T("\yVote: %s\w^n^n1.  %s^n2.  %s",players[numplayers]),
                   quest, g_optionName[0], g_optionName[1])
      }
      else {
        len = format(menu_msg, 255, _T("Vote: %s^n^n1.  %s^n2.  %s",players[numplayers]),
                   quest, g_optionName[0], g_optionName[1])
      }
      if(g_optionName[2][0]) {
        len += format(menu_msg[len], 255-len, "^n3.  %s", g_optionName[2])
        keys |= (1<<2)
      }
      if(g_optionName[3][0]) {
        len += format(menu_msg[len], 255-len, "^n4.  %s", g_optionName[3])
        keys |= (1<<3)
      }
      g_menumsg[players[numplayers]][0] = '^0'
      copy(g_menumsg[players[numplayers]], 255, menu_msg)
    }
    get_user_menu(players[numplayers], menuid, tempkeys)
    if(tempkeys == 0 || menuid < 0) {
      show_menu(players[numplayers], keys, menu_msg, floatround(vote_time), "Vote: ")
    }
    g_menukeys = keys
  }

  g_menutitle[0] = '^0'
  copy(g_menutitle, 15, "Vote: ")
  remove_task(1467851)
  new param[3]
  param[0] = 1
  param[1] = menuid_vote
  param[2] = get_cvar_num("amx_vote_time")
  set_task(CHECK_MENU_TIME, "check_menu", 1467851, param, 3)

  if(cvar_exists(quest) && (get_user_flags(id) & ADMIN_CVAR) && canChangeCvar(id, quest)) {
    g_execResult = true
    format(g_Answer, 127, "%s %%s", quest)
  }
  else if(command && ((playerflags & cmdflags) == cmdflags)) {
    g_execResult = true
    format(g_Answer, 127, "%s %%s", quest)
  }
  else {
    g_execResult = false
    format(g_Answer, 127, "%s - %%s", quest)
  }
  set_cvar_float("amx_last_voting", get_gametime() + vote_time)
  g_voteRatio = get_cvar_float("amx_vote_ratio")
  
  set_task(vote_time, "checkVotes", 99889988)
  console_print(id, _T(g_votingStarted))
  g_voteCount = {0, 0, 0, 0}
  g_yesNoVote = 0
  return PLUGIN_HANDLED
}

public cmdVoteKickBan(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new Float:voting = get_cvar_float("amx_last_voting")
  if(voting > get_gametime()) {
    console_print(id, _T(g_alreadyVoting))
    return PLUGIN_HANDLED
  }

  if(voting && voting + get_cvar_float("amx_vote_delay") > get_gametime()) {
    console_print(id, _T(g_notAllowed))
    return PLUGIN_HANDLED
  }

  new cmd[32]
  read_argv(0, cmd, 31)
  new voteban = equal(cmd, "amx_voteban")
  new arg[32]
  read_argv(1, arg, 31)
  new player = cmd_target(id, arg, 1)
  if(!player) return PLUGIN_HANDLED

  if(voteban && is_user_bot(player)) {
    new imname[32]
    get_user_name(player, imname, 31)
    console_print(id, _T("That action can't be performed on bot ^"%s^""), imname)
    return PLUGIN_HANDLED
  }

  for(new i = 1; i <= g_maxplayers; i++) {
    g_hasVoted[i] = false
  }

  g_voteCaller = id
  
  new keys = (1<<0)|(1<<1)
  new menu_msg[256]
  new players[32]
  new numplayers
  new Float:vote_time = get_cvar_float("amx_vote_time") + 2.0

  get_user_name(player, arg, 31)
  new menuid, tempkeys
  get_players(players, numplayers, "c")
  for(numplayers--; numplayers >= 0; numplayers--) {
    if(g_coloredMenus) {
      format(menu_msg, 255, _T("\y%s %s?\w^n^n1.  Yes^n2.  No",players[numplayers]),
      voteban ? _T("Ban",players[numplayers]) : _T("Kick",players[numplayers]), arg)
    }
    else {
      format(menu_msg, 255, _T("%s %s?^n^n1.  Yes^n2.  No", players[numplayers]),
      voteban ? _T("Ban",players[numplayers]) : _T("Kick",players[numplayers]), arg)
    }
    get_user_menu(players[numplayers], menuid, tempkeys)
    if(tempkeys == 0 || menuid < 0) {
      show_menu(players[numplayers], keys, menu_msg, floatround(vote_time), voteban ? "Ban " : "Kick ")
    }
    g_menumsg[players[numplayers]][0] = '^0'
    copy(g_menumsg[players[numplayers]], 255, menu_msg)
  }

  g_menukeys = keys
  g_menutitle[0] = '^0'
  copy(g_menutitle, 15, voteban ? "Ban " : "Kick ")
  remove_task(1467851)
  new param[3]
  param[0] = 1
  param[1] = voteban ? menuid_ban : menuid_kick
  param[2] = get_cvar_num("amx_vote_time")
  set_task(CHECK_MENU_TIME, "check_menu", 1467851, param, 3)

  g_yesNoVote = 1

  new banByIp = 0
  new arg2[32]
  read_argv(2, arg2, 31)
  if(equali(arg2, "ip"))
    banByIp = 1
  
  if(voteban) {
    if(banByIp)
      get_user_ip(player, g_optionName[0], 31, 1)
    else
      get_user_authid(player, g_optionName[0], 31)
  }
  else {
    num_to_str(get_user_userid(player), g_optionName[0], 31)
  }

  if(is_user_connected(id)) {
    new authid[32], name[32]
    get_user_authid(id, authid, 31)
    get_user_name(id, name, 31)
    log_to_file(g_logFile,"Vote: ^"%s<%d><%s><>^" vote %s (target ^"%s^")",name,get_user_userid(id),authid,voteban ? "ban" : "kick",arg)

    switch(get_cvar_num("amx_show_activity")) {
      case 2: {
        if(is_user_admin(id)) {
          client_print(0, print_chat, _T("%s %s: vote %s for %s"), _T(g_adminTag), name, voteban ? _T("ban") : _T("kick"), arg)
        }
        else {
          client_print(0, print_chat, _T("%s %s: vote %s for %s"), _T(g_playerTag), name, voteban ? _T("ban") : _T("kick"), arg)
        }
      }
      case 1: {
        if(is_user_admin(id)) {
          client_print(0, print_chat, _T("%s: vote %s for %s"), _T(g_adminTag), voteban ? _T("ban") : _T("kick"), arg)
        }
        else {
          client_print(0, print_chat, _T("%s: vote %s for %s"), _T(g_playerTag), voteban ? _T("ban") : _T("kick"), arg)
        }
      }
    }
  }

  g_execResult = true
  set_cvar_float("amx_last_voting", get_gametime() + vote_time)
  g_voteRatio = get_cvar_float(voteban ? "amx_voteban_ratio" : "amx_votekick_ratio")
  if(voteban) {
    if(banByIp)
      g_Answer = "addip 30.0 ^"%s^";wait;writeip"
    else
      g_Answer = "banid 30.0 %s kick"
  }
  else {
    g_Answer = "kick #%s"
  }
  set_task(vote_time, "checkVotes", 99889988)
  console_print(id, _T(g_votingStarted))
  g_voteCount = {0, 0, 0, 0}
  return PLUGIN_HANDLED
}
