/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

#include <translator>
#include <amxmod>
#include <amxmisc>

#define AUTOMATIC_PRECACHING

/* Commands Menus */

#define MAX_CMDS_LAYERS 3

new g_cmdMenuName[MAX_CMDS_LAYERS][] = {
  "Commands Menu",
  "Configs Menu",
  "Speech Menu"
}

// For the Translations to work
new g_cmdMenuName2[MAX_CMDS_LAYERS][] = {
  "Commands Menu",
  "Configs Menu",
  "Speech Menu"
}

new g_cmdMenuCmd[MAX_CMDS_LAYERS][] = {
  "amx_cmdmenu",
  "amx_cfgmenu",
  "amx_speechmenu"
}

new g_cmdMenuCfg[MAX_CMDS_LAYERS][] = {
  "cmds.ini",
  "configs.ini",
  "speech.ini"
}

new g_cmdMenuHelp[ MAX_CMDS_LAYERS][] = {
  "- displays commands menu",
  "- displays configs menu",
  "- displays speech menu"
}

/* End of Commands Menu */

#define MAX_CMDS    32
#define MAX_CVARS   64

#define MAX_PLUGINS 128
#define RESETCHAT_TIME 20.0

new g_cmdName[MAX_CMDS*MAX_CMDS_LAYERS][32]
new g_cmdCmd[MAX_CMDS*MAX_CMDS_LAYERS][64]
new g_cmdMisc[MAX_CMDS*MAX_CMDS_LAYERS][2]
new g_cmdNum[MAX_CMDS_LAYERS]

new g_cvarNames[MAX_CVARS][32]
new g_cvarMisc[MAX_CVARS][3]
new g_cvarCmd[MAX_CVARS*5][32]
new g_cvarCmdNum
new g_cvarNum

new g_menuType[33]
new g_selectedPlugin[33]
new g_Key[33]
new bool:g_getChat[33]

new g_menuPosition[33]
new g_menuSelect[33][MAX_PLUGINS]
new g_menuSelectNum[33]
new g_menuLayer[33]

new g_coloredMenus
new g_logFile[16]

public plugin_init() {
  load_translations("cmdmenu")
  register_plugin(_T("Commands Menu"),AMX_VERSION,"AMX Team")

  new configdir[64], workdir[128]
  build_path(configdir, 63, "$configdir")
  for(new a = 0; a < MAX_CMDS_LAYERS; ++a) {
    register_menucmd(register_menuid(g_cmdMenuName[a]),1023,"actionCmdMenu")
    register_clcmd(g_cmdMenuCmd[a],"cmdCmdMenu",ADMIN_MENU,_T(g_cmdMenuHelp[a]))
    format(workdir, 127, "%s/%s", configdir, g_cmdMenuCfg[a])
    loadCmdSettings(workdir, a)
  }

  register_menucmd(register_menuid("Cvars Menu"),1023,"actionCvarMenu")
  register_clcmd("amx_cvarmenu","cmdCvarMenu",ADMIN_CVAR,_T("- displays cvars menu"))
  format(workdir, 127, "%s/cvars.ini", configdir)
  loadCvarSettings(workdir)

  register_menucmd(register_menuid("Plugins Menu"),1023,"actionPluginsMenu")
  register_menucmd(register_menuid("Plugins CmdMenu"),1023,"actionPluginsCmdMenu")
  register_clcmd("amx_plugcmdmenu","cmdPluginsCmdMenu",ADMIN_MENU,_T("[filename.amx / plugin name] - displays plugins commands menu"))
  register_menucmd(register_menuid("Plugins CvarMenu"),1023,"actionPluginsCvarMenu")
  register_clcmd("amx_plugcvarmenu","cmdPluginsCvarMenu",ADMIN_CVAR,_T("[filename.amx / plugin name] - displays plugins cvars menu"))
  register_clcmd("say","SayEvent")

  g_coloredMenus = colored_menus()
  get_logfile(g_logFile, 15)
}

canChangeCvar(id, const cvar[]) {
  if(equali(cvar, "amx_version")
  || equali(cvar, "fun_version")
  || equali(cvar, "amx_last_voting"))
    return 0
  if(!(get_user_flags(id) & ADMIN_RCON) &&
  (  equali(cvar, "sv_cheats")
  || equali(cvar, "sv_password")
  || equali(cvar, "rcon_password")
  || equali(cvar, "amx_show_activity")
  || equali(cvar, "amx_mode")
  || equali(cvar, "amx_password_field")
  || equali(cvar, "amx_default_access")
  || equali(cvar, "amx_reservation")
  || equali(cvar, "amx_debug")
  || equali(cvar, "amx_langdebug")
  || equali(cvar, "csstats_rank")
  || equali(cvar, "csstats_rankbots")
  || equali(cvar, "csstats_pause")
  || equali(cvar, "csstats_maxsize")
  || equali(cvar, "csstats_reset")
  || equali(cvar, "dodstats_rank")
  || equali(cvar, "dodstats_rankbots")
  || equali(cvar, "dodstats_pause")
  || equali(cvar, "dodstats_maxsize")
  || equali(cvar, "dodstats_reset")))
    return 0
  return 1
}

basic_command(const cmd[]) {
  if(equali(cmd, "buy")
  || equali(cmd, "buyequip")
  || equali(cmd, "buyammo1")
  || equali(cmd, "buyammo2")
  || equali(cmd, "cl_setautobuy")
  || equali(cmd, "cl_autobuy")
  || equali(cmd, "cl_setrebuy")
  || equali(cmd, "cl_rebuy")
  || equali(cmd, "say")
  || equali(cmd, "say_team")
  || equali(cmd, "chooseteam")
  || equali(cmd, "drop")
  || equali(cmd, "nightvision")
  || equali(cmd, "lastinv")
  || equali(cmd, "invnext")
  || equali(cmd, "invprev")
  || equali(cmd, "slot", 4)
  || equali(cmd, "radio", 5)
  || equali(cmd, "amxauth", 7)
  || equali(cmd, "pauseAck")
  || equali(cmd, "amxres", 6))
    return 1
  return 0
}

public client_connect(id) {
  g_getChat[id] = false
}

/* Commands menu */

public actionCmdMenu(id,key) {
  switch(key) {
    case 8: displayCmdMenu(id, ++g_menuPosition[id])
    case 9: displayCmdMenu(id, --g_menuPosition[id])
    default: {
      new option = g_menuSelect[id][g_menuPosition[id] * 8 + key]
      new flags = g_cmdMisc[option][1]
      if(flags & 1)
        server_cmd(g_cmdCmd[option])
      else if(flags & 2) {
        new cmd[32], cmdaccess[32], playerflags, cmdflags, noaccess = 0
        playerflags = get_user_flags(id)
        parse(g_cmdCmd[option], cmd, 31)
        if(get_cmdaccess(cmd, cmdaccess, 31)) {
          cmdflags = read_flags(cmdaccess)
          if((playerflags & cmdflags) != cmdflags) noaccess = 1
          if(noaccess) {
            set_user_flags(id, cmdflags)
          }
          client_cmd(id, g_cmdCmd[option])
          if(noaccess) {
            new param[3]
            param[0] = id
            param[1] = playerflags
            set_task(0.3, "setOldFlags", 800+cmdflags+id, param, 3)
          }
        }
        else client_cmd(id, g_cmdCmd[option])
      }
      else if(flags & 4)
        client_cmd(0, g_cmdCmd[option])
      if(flags & 8) displayCmdMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

public setOldFlags(param[]) {
  new id = param[0]
  new playerflags = param[1]
  if(is_user_connected(id)) {
    remove_user_flags(id)
    set_user_flags(id, playerflags)
  }
}

displayCmdMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0
  new start = pos * 8

  if(start >= g_menuSelectNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
  "\y%s\R%d/%d^n\w^n" : "%s %d/%d^n^n", _T(g_cmdMenuName2[g_menuLayer[id]],id), 
    pos+1, (g_menuSelectNum[id] / 8 + ((g_menuSelectNum[id] % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)

  if(end > g_menuSelectNum[id])
    end = g_menuSelectNum[id]

  for(new a = start; a < end; ++a) {
    if(g_cmdCmd[g_menuSelect[id][a]][0] == '-') {
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%s^n\w", g_cmdName[g_menuSelect[id][a]])
      else
        len += format(menuBody[len], 511-len, "%s^n", g_cmdName[g_menuSelect[id][a]])
      ++b
    }
    else {
      keys |= (1<<b)
      len += format(menuBody[len], 511-len, "%d. %s^n", ++b, is_translated(g_cmdName[g_menuSelect[id][a]]) ? _T(g_cmdName[g_menuSelect[id][a]],id) : g_cmdName[g_menuSelect[id][a]])
    }
  }

  if(end != g_menuSelectNum[id]) {
    format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else format(menuBody[len], 511-len, "^n0. %s", pos ? _T("Back",id) : _T("Exit",id))
  show_menu(id, keys, menuBody, -1, g_cmdMenuName[g_menuLayer[id]])
}

public cmdCmdMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  new szCmd[32]
  read_argv(0, szCmd, 31)

  new lvl = 0

  while(lvl < MAX_CMDS_LAYERS) {
    if(equal(g_cmdMenuCmd[lvl], szCmd))
      break
    ++lvl
  }

  g_menuLayer[id] = lvl

  new flags = get_user_flags(id)

  g_menuSelectNum[id] = 0

  new a = lvl * MAX_CMDS
  new d , c = 0

  while(c < g_cmdNum[lvl]) {
    d = a + c

    if((g_cmdMisc[d][0] & flags) == g_cmdMisc[d][0]) {
      g_menuSelect[id][g_menuSelectNum[id]++] = d
    }

    ++c
  }

  displayCmdMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

loadCmdSettings(szFilename[],level) {
  if(!file_exists(szFilename))
    return 0

  g_cmdNum[level] = 0

  new text[256], szFlags[32], szAccess[32]
  new a, pos = 0, c, d = level * MAX_CMDS

  while(g_cmdNum[level] < MAX_CMDS && (pos = read_file(szFilename, pos, text, 255, a))) {
    if(!a || text[0] == ';' || text[0] == '/') continue

    c = d + g_cmdNum[level]

    if(parse(text, g_cmdName[c], 31, g_cmdCmd[c], 63, szFlags, 31, szAccess, 31) > 3) {
      replace_all(g_cmdCmd[c], 63, "\'", "^"")

      g_cmdMisc[c][1] = read_flags(szFlags)
      g_cmdMisc[c][0] = read_flags(szAccess)
      g_cmdNum[level]++
    }
  }
  return 1
}

/* Cvars menu */

public actionCvarMenu(id,key) {
  switch(key) {
    case 8: displayCvarMenu(id, ++g_menuPosition[id])
    case 9: displayCvarMenu(id, --g_menuPosition[id])
    default: {
      new option = g_menuSelect[id][g_menuPosition[id] * 8 + key]

      new szValue[32]
      get_cvar_string(g_cvarNames[option], szValue, 31)

      new end = g_cvarMisc[option][2]
      new start = g_cvarMisc[option][1]

      for(new i = start ; ; ++i) {
        if(i < end) {
          if(equal(szValue, g_cvarCmd[i])) {
            if(++i >= end) {
              i = start
            }
            set_cvar_string(g_cvarNames[option], g_cvarCmd[i])
            break
          }
        }
        else {
          set_cvar_string(g_cvarNames[option], g_cvarCmd[start])
          break
        }
      }
      displayCvarMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displayCvarMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0
  new start = pos * 8

  if(start >= g_menuSelectNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\yCvars Menu\R%d/%d^n\w^n",id) : _T("Cvars Menu %d/%d^n^n",id),
    pos+1, (g_menuSelectNum[id] / 8 + ((g_menuSelectNum[id] % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)
  new szValue[64]

  if(end > g_menuSelectNum[id])
    end = g_menuSelectNum[id]

  for(new a = start; a < end; ++a) {
    get_cvar_string(g_cvarNames[g_menuSelect[id][a]], szValue, 31)
    keys |= (1<<b)
    ++b
    if(g_coloredMenus)
      len += format(menuBody[len], 511-len, "%d. %s\R%s^n\w", b, g_cvarNames[g_menuSelect[id][a]], szValue)
    else
      len += format(menuBody[len], 511-len, "%d. %s    %s^n", b, g_cvarNames[g_menuSelect[id][a]], szValue)
  }

  if(end != g_menuSelectNum[id]) {
    format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else format(menuBody[len], 511-len, "^n0. %s", pos ? _T("Back",id) : _T("Exit",id))
  show_menu(id, keys, menuBody, -1, "Cvars Menu")
}

public cmdCvarMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  new flags = get_user_flags(id)

  g_menuSelectNum[id] = 0

  for(new a = 0; a < g_cvarNum; ++a)
    if((g_cvarMisc[a][0] & flags) == g_cvarMisc[a][0])
      g_menuSelect[id][g_menuSelectNum[id]++] = a

  displayCvarMenu(id, g_menuPosition[id] = 0)

  return PLUGIN_HANDLED
}

loadCvarSettings(szFilename[]) {
  if(!file_exists(szFilename))
    return 0

  g_cvarNum = 0

  new text[256], szValues[12][32]
  new inum, a, pos = 0
  new cvar_values = MAX_CVARS * 5

  while(g_cvarNum < MAX_CVARS && (pos = read_file(szFilename, pos, text, 255, a))) {
    if(!a || text[0] == ';' || text[0] == '/') continue

    inum = parse(text, g_cvarNames[g_cvarNum], 31,
      szValues[0], 31, szValues[1], 31 ,szValues[2], 31,
      szValues[3], 31, szValues[4], 31 ,szValues[5], 31,
      szValues[6], 31, szValues[7], 31 ,szValues[8], 31,
      szValues[9], 31, szValues[10], 31 ,szValues[11], 31)

    inum -= 2
    if(inum < 2) continue

    g_cvarMisc[g_cvarNum][1] = g_cvarCmdNum

    for(a = 0; a < inum && g_cvarCmdNum < cvar_values; ++a) {
        replace_all(szValues[a], 31, "\'", "^"")
        copy(g_cvarCmd[g_cvarCmdNum], 31, szValues[a])
        g_cvarCmdNum++
    }

    g_cvarMisc[g_cvarNum][2] = g_cvarCmdNum
    g_cvarMisc[g_cvarNum][0] = read_flags(szValues[inum])
    g_cvarNum++
  }
  return 1
}

/* Plugins Cmds & Cvars menu */

public SayEvent(id) {
  if(g_getChat[id]) {
    new args[128]
    read_args(args, 127)
    if(containi(args, "resetchat") != -1) {
      client_print(id, print_chat, _T("[AMX] You can now use the ^"say^" command normally."))
      g_getChat[id] = false
      return PLUGIN_HANDLED
    }
    new filename[64], unused[2], status[2], jit
    get_plugin(g_selectedPlugin[id],filename,63,unused,0,unused,0,unused,0,status,1,jit,id)
    if(status[0] == 'r') {
      if(g_menuType[id] == 0) {
        new cmd[32], info[1]
        get_plugincmd(filename,g_menuSelect[id][g_menuPosition[id] * 4 + g_Key[id]],cmd,31,info,0,info,0,id,3)
        remove_quotes(args)
        new command[128]
        format(command, 127, "%s %s", cmd, args)
        client_cmd(id, command)
      }
      else if(g_menuType[id] == 1) {
        new cvar[32], value[32]
        get_plugincvar(filename,g_menuSelect[id][g_menuPosition[id] * 8 + g_Key[id]],cvar,31,value,31)
        replace_all(args, 127, "^"", "")

        new authid[32], admin_name[32]
        get_user_authid(id, authid, 31)
        get_user_name(id, admin_name, 31)
        log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" set cvar (name ^"%s^") (value ^"%s^")",
          admin_name,get_user_userid(id),authid,cvar,args)
        set_cvar_string(cvar, args)

        new activity = get_cvar_num("amx_show_activity")
        if(activity) {
          if(activity == 1) {
            if(equal(cvar, "rcon_password") || equal(cvar, "sv_password"))
              client_print(0, print_chat, _T("ADMIN: set cvar %s to ^"%s^""), cvar, _T("*** PROTECTED ***"))
            else
              client_print(0, print_chat, _T("ADMIN: set cvar %s to ^"%s^""), cvar, args)
          }
          else {
            if(equal(cvar, "rcon_password") || equal(cvar, "sv_password"))
              client_print(0, print_chat, _T("ADMIN %s: set cvar %s to ^"%s^""), admin_name, cvar, _T("*** PROTECTED ***"))
            else
              client_print(0, print_chat, _T("ADMIN %s: set cvar %s to ^"%s^""), admin_name, cvar, args)
          }
        }

        console_print(id, _T("Cvar ^"%s^" changed to ^"%s^""), cvar, args)
      }
    }
    remove_task(7534812+id)
    g_getChat[id] = false
    return PLUGIN_HANDLED
  }
  return PLUGIN_CONTINUE
}

public resetGetChat(id) {
  id -= 7534812
  if(g_getChat[id]) g_getChat[id] = false
  client_print(id, print_chat, _T("[AMX] You can now use the ^"say^" command normally."))
}

public actionPluginsCmdMenu(id,key) {
  switch(key) {
    case 8: displayPluginsCmdMenu(id, ++g_menuPosition[id])
    case 9: displayPluginsCmdMenu(id, --g_menuPosition[id])
    default: {
      g_Key[id] = key
      g_getChat[id] = true
      client_cmd(id, "messagemode")
      remove_task(7534812+id)
      set_task(RESETCHAT_TIME, "resetGetChat", 7534812+id)
    }
  }
  return PLUGIN_HANDLED
}

displayPluginsCmdMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0
  new start = pos * 4

  if (start >= g_menuSelectNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
  _T("\yPlugins Commands Menu\R%d/%d^n\w^n",id) : _T("Plugins Commands Menu %d/%d^n^n",id), 
    pos+1, (g_menuSelectNum[id] / 4 + ((g_menuSelectNum[id] % 4) ? 1 : 0)))

  new end = start + 4
  new keys = (1<<9)

  if(end > g_menuSelectNum[id])
    end = g_menuSelectNum[id]

  for(new a = start; a < end; ++a) {
    new filename[64], unused[2], jit, cmd[32], accessflags[32], info[76]
    get_plugin(g_selectedPlugin[id],filename,63,unused,0,unused,0,unused,0,unused,0,jit,id)
    get_plugincmd(filename,g_menuSelect[id][a],cmd,31,accessflags,31,info,75,id,3)

    new flags = get_user_flags(id)
    new cmdFlags = read_flags(accessflags)

    if((flags & cmdFlags) == cmdFlags || !cmdFlags) {
      keys |= (1<<b)
      info[75-strlen(cmd)] = '^0'
      len += format(menuBody[len], 511-len, "%d. %s:%s^n", ++b, cmd, info)
    }
    else {
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%s^n\w", cmd)
      else
        len += format(menuBody[len], 511-len, "%s^n", cmd)
      ++b
    }
  }

  len += format(menuBody[len], 511-len, _T("^nAfter choosing a command, type the arguments in the chat.^n",id))
  len += format(menuBody[len], 511-len, _T("Type ^"resetchat^" to cancel.^n",id))

  if(end != g_menuSelectNum[id]) {
    format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else format(menuBody[len], 511-len, "^n0. %s", pos ? _T("Back",id) : _T("Exit",id))
  show_menu(id, keys, menuBody, -1, "Plugins CmdMenu")
}

public cmdPluginsCmdMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1)) 
    return PLUGIN_HANDLED

  new arg[64]
  read_argv(1, arg, 63)
  new pluginNum = -1

  new flags = get_user_flags(id)

  g_menuSelectNum[id] = 0

  new nump = get_pluginsnum()
  new filename[64], name[64], unused[2], status[2], jit, cmd[32], accessflags[32], info[1]
  new numcmd
  new cmdFlags
  for(new i = 0; i < nump; ++i) {
    get_plugin(i,filename,63,name,63,unused,0,unused,0,status,1,jit,id)
    if(status[0] == 'r') {
      numcmd = get_plugincmdsnum(filename, 3)
      for(new j = 0; j < numcmd; j++) {
        get_plugincmd(filename,j,cmd,31,accessflags,31,info,0,-1,3)
        cmdFlags = read_flags(accessflags)
        if(((flags & cmdFlags) == cmdFlags || !cmdFlags) && !basic_command(cmd)) {
          g_menuSelect[id][g_menuSelectNum[id]++] = i
          if(equali(filename, arg) || equali(name, arg)) pluginNum = g_menuSelectNum[id] - 1
          break
        }
      }
    }
  }
  g_menuType[id] = 0
  if(pluginNum != -1) {
    g_menuPosition[id] = pluginNum / 8
    actionPluginsMenu(id, pluginNum % 8)
  }
  else displayPluginsMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

public actionPluginsCvarMenu(id,key) {
  switch(key) {
    case 8: displayPluginsCvarMenu(id, ++g_menuPosition[id])
    case 9: displayPluginsCvarMenu(id, --g_menuPosition[id])
    default: {
      g_Key[id] = key
      g_getChat[id] = true
      client_cmd(id, "messagemode")
      remove_task(7534812+id)
      set_task(RESETCHAT_TIME, "resetGetChat", 7534812+id)
    }
  }
  return PLUGIN_HANDLED
}

displayPluginsCvarMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0
  new start = pos * 8

  if(start >= g_menuSelectNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
  _T("\yPlugins Cvars Menu\R%d/%d^n\w^n",id) : _T("Plugins Cvars Menu %d/%d^n^n",id), 
    pos+1, (g_menuSelectNum[id] / 8 + ((g_menuSelectNum[id] % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)

  if(end > g_menuSelectNum[id])
    end = g_menuSelectNum[id]

  new filename[64], unused[2], jit, cvar[32], value[32]
  for(new a = start; a < end; ++a) {
    get_plugin(g_selectedPlugin[id],filename,63,unused,0,unused,0,unused,0,unused,0,jit,id)
    get_plugincvar(filename,g_menuSelect[id][a],cvar,31,value,31)

    if(canChangeCvar(id, cvar)) {
      keys |= (1<<b)
      len += format(menuBody[len], 511-len, "%d. %s ( %s )^n", ++b, cvar, value)
    }
    else {
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%s^n\w", cvar)
      else
        len += format(menuBody[len], 511-len, "%s^n", cvar)
      ++b
    }
  }

  len += format(menuBody[len], 511-len, _T("^nAfter choosing a cvar, type the new value in the chat.^n",id))
  len += format(menuBody[len], 511-len, _T("Type ^"resetchat^" to cancel.^n",id))

  if(end != g_menuSelectNum[id]) {
    format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else format(menuBody[len], 511-len, "^n0. %s", pos ? _T("Back",id) : _T("Exit",id))
  show_menu(id, keys, menuBody, -1, "Plugins CvarMenu")
}

public cmdPluginsCvarMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1)) 
    return PLUGIN_HANDLED

  new arg[64]
  read_argv(1, arg, 63)
  new pluginNum = -1

  g_menuSelectNum[id] = 0

  new nump = get_pluginsnum()
  new filename[64], name[64], unused[2], status[2], jit
  for(new i = 0; i < nump; ++i) {
    get_plugin(i,filename,63,name,63,unused,0,unused,0,status,1,jit,id)
    if(status[0] == 'r') {
      if(get_plugincvarsnum(filename) > 0) {
        g_menuSelect[id][g_menuSelectNum[id]++] = i
        if(equali(filename, arg) || equali(name, arg)) pluginNum = g_menuSelectNum[id] - 1
        continue
      }
    }
  }
  g_menuType[id] = 1
  if(pluginNum != -1) {
    g_menuPosition[id] = pluginNum / 8
    actionPluginsMenu(id, pluginNum % 8)
  }
  else displayPluginsMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

public actionPluginsMenu(id,key) {
  switch(key) {
    case 8: displayPluginsMenu(id, ++g_menuPosition[id])
    case 9: displayPluginsMenu(id, --g_menuPosition[id])
    default: {
      g_selectedPlugin[id] = g_menuSelect[id][g_menuPosition[id] * 8 + key]
      if(g_menuType[id] == 0) {
        new flags = get_user_flags(id)
        g_menuSelectNum[id] = 0
        new filename[64], unused[2], status[2], jit, cmd[32], accessflags[32], info[1]
        get_plugin(g_selectedPlugin[id],filename,63,unused,0,unused,0,unused,0,status,1,jit,id)
        if(status[0] == 'r') {
          new numcmd = get_plugincmdsnum(filename,3)
          for(new j = 0; j < numcmd; j++) {
            get_plugincmd(filename,j,cmd,31,accessflags,31,info,0,-1,3)
            new cmdFlags = read_flags(accessflags)
            if(((flags & cmdFlags) == cmdFlags || !cmdFlags) && !basic_command(cmd)) {
              g_menuSelect[id][g_menuSelectNum[id]++] = j
            }
          }
          if(g_menuSelectNum[id] > 0) displayPluginsCmdMenu(id, g_menuPosition[id] = 0)
        }
      }
      else if(g_menuType[id] == 1) {
        g_menuSelectNum[id] = 0
        new filename[64], unused[2], status[2], jit
        get_plugin(g_selectedPlugin[id],filename,63,unused,0,unused,0,unused,0,status,1,jit,id)
        if(status[0] == 'r') {
          new numcvar = get_plugincvarsnum(filename)
          for(new j = 0; j < numcvar; j++) {
            g_menuSelect[id][g_menuSelectNum[id]++] = j
          }
          if(g_menuSelectNum[id] > 0) displayPluginsCvarMenu(id, g_menuPosition[id] = 0)
        }
      }
    }
  }
  return PLUGIN_HANDLED
}

displayPluginsMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0
  new start = pos * 8

  if(start >= g_menuSelectNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
  _T("\yPlugins Menu\R%d/%d^n\w^n",id) : _T("Plugins Menu %d/%d^n^n",id), 
    pos+1, (g_menuSelectNum[id] / 8 + ((g_menuSelectNum[id] % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)

  if(end > g_menuSelectNum[id])
    end = g_menuSelectNum[id]

  new name[64], unused[2], status[2], jit
  for(new a = start; a < end; ++a) {
    get_plugin(g_menuSelect[id][a],unused,0,name,63,unused,0,unused,0,status,1,jit,id)
    if(status[0] == 'r') {
      keys |= (1<<b)
      len += format(menuBody[len], 511-len, "%d. %s^n",++b, name)
    }
    else {
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%s^n\w", name)
      else
        len += format(menuBody[len], 511-len, "%s^n", name)
      ++b
    }
  }
  if(end != g_menuSelectNum[id]) {
    format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else format(menuBody[len], 511-len, "^n0. %s", pos ? _T("Back",id) : _T("Exit",id))
  show_menu(id, keys, menuBody, -1, "Plugins Menu")
}

#if defined AUTOMATIC_PRECACHING
public plugin_precache() {
  new szFilename[64]
  build_path(szFilename, 63, "$configdir/speech.ini")
  if(!file_exists(szFilename))
    return 0

  new text[256], szName[32], szSound[128], szFlags[32], szAccess[32]
  new num = 0, a, pos = 0
  new ext[5]
  new arg1[32], arg2[64], arg3[64]
  
  while(num < MAX_CMDS && (pos = read_file(szFilename, pos, text, 255, a))) {
    if(!a || text[0] == ';' || text[0] == '/') continue

    if(parse(text, szName, 31, szSound, 127, szFlags, 31, szAccess, 31) > 3) {
      new parsedNums = parse(szSound, arg1, 31, arg2, 63, arg3, 63)
      if(parsedNums == 2 && equali(arg1, "spk")) {
        copy(szSound, 127, arg2)
        copy(ext, 4, ".wav")
      }
      else if(parsedNums == 3 && equali(arg1, "mp3") && (equali(arg2, "play") || equali(arg2, "loop"))) {
        copy(szSound, 127, arg3)
        copy(ext, 4, ".mp3")
      }
      else continue

      replace_all(szSound, 127, "\'", "")

      if(szSound[0] == '/') replace(szSound, 127, "/", "")

      if(ext[1] == 'm' || (!equali(szSound, "vox", 3) && !equali(szSound, "barney", 6) && !equali(szSound, "hgrunt", 6))) {
        new len = strlen(szSound)
        if(len > 3 && !equali(szSound[len-4], ext)) add(szSound, 127, ext)
        if(ext[1] == 'w') format(szSound, 127, "sound/%s", szSound)
        if(file_exists(szSound)) {
          if(ext[1] == 'm') {
            precache_generic(szSound)
          }
          else {
            replace(szSound, 127, "sound/", "")
            precache_sound(szSound)
          }
        }
      }
      num++
    }
  }
  return 0
}
#endif
