/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 *
 * Custom Map Configs
 *  Set map-specific variables and commands
 *
 * by JustinHoMi (justin@justinmitchell.net)
 *  http://www.modkillers.com
 *  #modkillers in irc.gamesnet.net
 *
 * Place your custom configs in the designated folder
 *  (addons/amx/config/maps by default)
 * Configs should be in the format "mapname.cfg"
 * Insert any cvar or cmd to be executed at map change
 *
 * Changelog:
 *  2006.1 - General map config files should be named like this: prefix_xxx.cfg
 *           For example: prefix_de.cfg, prefix_scoutzknivez.cfg, prefix_cs_.cfg
 *  2005.1 - Added option to load general map config like "cs_.cfg", "de_.cfg", "as_.cfg"
 *           (by _KaszpiR_, fixed by KRoT@L)
 *  0.9.9  - Added translations support for AMX Mod 0.9.9
 *  0.61   - Changes load delay to 6s (to work better with SQL ServerCfg)
 *  0.6    - Execs configs rather than loading file
 *         - Delays execution for 5s after map changes
 *  0.5    - Initial release
 *
 */

#include <translator>
#include <amxmod>
#include <amxmisc>

new currentmap[32]

public plugin_init() {
  load_translations("mapconfig")
  register_plugin(_T("Custom Map Configs"),AMX_VERSION,"AMX Team")

  new filename[128], filepath[128], prefixfile[64]

  build_path(filepath, 127, "$configdir/maps")
  get_mapname(currentmap, 31)
  
  new len, len2, pos = 2
  while((pos = read_dir(filepath, pos, prefixfile, 63, len))) {
    if(len <= 7 || (len > 7 && (!equali(prefixfile, "prefix_", 7) || !equali(prefixfile[len-4], ".cfg", 4)))) continue
    replace(prefixfile[7], 120, ".cfg", "")
    len2 = strlen(prefixfile[7])
    if(equal(currentmap, prefixfile[7], len2)) {
      len = format(filename, 127, "%s/%s.cfg", filepath, prefixfile)
      set_task(6.1, "delayed_load", 64821379, filename, len+1)
      break
    }
  }

  filename[0] = '^0'
  len = format(filename, 127, "%s/%s.cfg", filepath, currentmap)

  if(file_exists(filename)) {
    set_task(6.5, "delayed_load", 46827913, filename, len+1)
  }
}

public delayed_load(filename[]) {
  server_print(_T("Loading custom map config for %s: %s"), currentmap, filename)
  server_cmd("exec %s", filename)
  server_exec()
}
