/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 *
 * Content for configuration copied from ClanMod cfg. file.
 *
 * Admin command:
 * amx_mapmenu - displays maps menu
 *
 * Example configuration in addons/amx/config/maps.ini

de_airstrip  "Airstrip - Bomb/Defuse"
de_aztec     "Aztec - Bomb/Defuse"
de_cbble     "Cobble - Bomb/Defuse"
de_chateau   "Chateau - Bomb/Defuse"
de_dust      "Dust - Bomb/Defuse"
de_dust2     "Dust II - Bomb/Defuse"
de_inferno   "Inferno - Bomb/Defuse"
de_nuke      "Nuke - Bomb/Defuse"
de_piranesi  "Piranesi - Bomb/Defuse"
de_prodigy   "Prodigy - Bomb/Defuse"
de_storm     "Storm - Bomb/Defuse"
de_survivor  "Survivor - Bomb/Defuse"
de_torn      "Torn - Bomb/Defuse"
de_train     "Trainyard - Bomb/Defuse"
de_vegas     "Vegas - Bomb/Defuse"
de_vertigo   "Vertigo - Bomb/Defuse"
cs_747       "747 Hijack - Hostage Rescue"
cs_assault   "Assault - Hostage Rescue"
cs_backalley "Alleyway - Hostage Rescue"
cs_estate    "Zaphod's Estate - Hostage Rescue"
cs_havana    "Havana - Hostage Rescue"
cs_italy     "Italy - Hostage Rescue"
cs_militia   "Militia - Hostage Rescue"
cs_office    "The Office Complex - Hostage Rescue"
cs_siege     "Canyon Siege - Hostage Rescue"
as_oilrig    "OilRig - Assassination"

 *
 */

#include <translator>
#include <amxmod>
#include <amxmisc>

#define CHECK_MENU_TIME 0.5

#define MAX_MAPS 128

new g_mapName[MAX_MAPS][32]
new g_mapDesc[MAX_MAPS][32]
new g_mapsNum
new g_menuPosition[33]
new g_logFile[16]

new g_voteCount[5]

new g_resultAck[] = "Result accepted"
new g_resultRef[] = "Result refused"

new g_voteSelected[33][4]
new g_voteSelectedNum[33]

new g_coloredMenus
new g_maxplayers

new g_yesNoVote
new g_choosed
new g_voteCaller
new bool:g_hasVoted[33]
new g_menukeys[33]
new g_menumsg[33][256]
new g_menutitle[32]

new menuid_whichmap, menuid_changemap

public plugin_init() {
  load_translations("mapsmenu")
  register_plugin(_T("Maps Menu"),AMX_VERSION,"AMX Team")
  register_clcmd("amx_mapmenu","cmdMapsMenu",ADMIN_MAP,_T("- displays changelevel menu"))
  register_clcmd("amx_votemapmenu","cmdVoteMapMenu",ADMIN_VOTE,_T("- displays votemap menu"))
  register_cvar("amx_mapsmenu_mapsloc","0")
  register_menucmd(register_menuid("Changelevel Menu"),1023,"actionMapsMenu")
  menuid_whichmap = register_menuid("Which map do you want?")
  menuid_changemap = register_menuid("Change map to")
  register_menucmd(menuid_whichmap,783,"voteCount")
  register_menucmd(menuid_changemap,515,"voteCount")
  register_menucmd(register_menuid("Votemap Menu"),1023,"actionVoteMapMenu")
  register_menucmd(register_menuid("The winner: "),3,"actionResult")

  get_logfile(g_logFile, 15)
  g_coloredMenus = colored_menus()
  g_maxplayers = get_maxplayers()
}

public plugin_cfg() {
  new filename[64]
  build_path(filename, 63, "$configdir/maps.ini")

  if(get_cvar_num("amx_mapsmenu_mapsloc") == 0) {
    if(loadSettings(filename)) {
      log_to_file(g_logFile,_T("Maps Menu: maps.ini succesfully loaded."))
    }
    else {
      log_to_file(g_logFile,_T("Maps Menu: Failed to load maps.ini or no maps inside the file. Trying to load mapcycle."))
      get_cvar_string("mapcyclefile", filename, 63)
      if(loadSettings(filename)) {
        log_to_file(g_logFile,_T("Maps Menu: mapcycle succesfully loaded."))
      }
      else {
        log_to_file(g_logFile,_T("Maps Menu: Failed to load mapcycle or no maps inside the file. Trying to load maps folder."))
        if(loadMapsFolder()) {
          log_to_file(g_logFile,_T("Maps Menu: maps folder succesfully loaded."))
        }
        else {
          log_to_file(g_logFile,_T("Maps Menu: Failed to load maps folder. No maps loaded."))
        }
      }
    }
  }
  else if(get_cvar_num("amx_mapsmenu_mapsloc") == 1) {
    get_cvar_string("mapcyclefile", filename, 63)
    if(loadSettings(filename)) {
      log_to_file(g_logFile,_T("Maps Menu: mapcycle succesfully loaded."))
    }
    else {
      log_to_file(g_logFile,_T("Maps Menu: Failed to load mapcycle or no maps inside the file. Trying to load maps folder."))
      if(loadMapsFolder()) {
        log_to_file(g_logFile,_T("Maps Menu: maps folder succesfully loaded."))
      }
      else {
        log_to_file(g_logFile,_T("Maps Menu: Failed to load maps folder. No maps loaded."))
      }
    }
  }
  else if(get_cvar_num("amx_mapsmenu_mapsloc") == 2) {
    set_cvar_num("amx_nominfromfile", 0)
    if(loadMapsFolder()) {
      log_to_file(g_logFile,_T("Maps Menu: maps folder succesfully loaded."))
    }
    else {
      log_to_file(g_logFile,_T("Maps Menu: Failed to load maps folder. No maps loaded."))
    }
  }
}

public check_menu(param[]) {
  new iter = param[0]
  new Float:vote_time = float(param[2]) + 2.0 - (iter * CHECK_MENU_TIME)
  new menutime = floatround(vote_time, floatround_floor)
  if(menutime > 0) {
    new players[32], numplayers
    new menuid, keys
    get_players(players, numplayers, "c")
    for(numplayers--; numplayers >= 0; numplayers--) {
      if(g_hasVoted[players[numplayers]]) continue
      get_user_menu(players[numplayers], menuid, keys)
      if(keys == 0 || menuid < 0) {
        show_menu(players[numplayers], g_menukeys[players[numplayers]], g_menumsg[players[numplayers]], menutime, g_menutitle)
      }
      else {
        if(menuid != param[1]) {
          if(menutime <= 6) {
            show_menu(players[numplayers], g_menukeys[players[numplayers]], g_menumsg[players[numplayers]], menutime, g_menutitle)
          }
        }
        else if(players[numplayers] == g_voteCaller) {
          show_menu(players[numplayers], g_menukeys[players[numplayers]], g_menumsg[players[numplayers]], menutime, g_menutitle)
        }
      }
    }
    ++param[0]
    set_task(CHECK_MENU_TIME, "check_menu", 1467852, param, 3)
  }
}

public autoRefuse() {
  log_to_file(g_logFile,"Vote: %s",g_resultRef)
  client_print(0, print_chat, _T(g_resultRef))
}

public actionResult(id,key) {
  remove_task(4545454)
  switch(key) {
    case 0: {
      new modName[8]
      get_modname(modName, 7)
      if(!equal(modName, "zp")) {
        message_begin(MSG_ALL, SVC_INTERMISSION)
        message_end()
      }
      set_task(2.0, "delayedChange", 33119977, g_mapName[g_choosed], strlen(g_mapName[g_choosed]) + 1)
      log_to_file(g_logFile,"Vote: %s",g_resultAck)
      client_print(0, print_chat, _T(g_resultAck))
    }
    case 1:
      autoRefuse()
  }
  return PLUGIN_HANDLED
}

new g_voteSuccess[] = "Voting successful. Map will be changed to"
new g_VoteFailed[] = "Voting failed"

public checkVotes(id) {
  remove_task(1467852)
  for(new i = 1; i <= g_maxplayers; i++) {
    g_hasVoted[i] = true
  }

  id -= 34567
  new num, ppl[32], a = 0
  get_players(ppl, num, "c")
  if(num == 0) num = 1
  g_choosed = -1
  for(new i = 0; i < g_voteSelectedNum[id]; ++i) {
    if(g_voteCount[a] < g_voteCount[i]) {
      a = i
    }
  }
  if(100 * g_voteCount[a] / num > 50) {
    g_choosed = g_voteSelected[id][a]
    client_print(0, print_chat, "%s %s", _T(g_voteSuccess), g_mapName[g_choosed])
    log_to_file(g_logFile,"Vote: %s %s",g_voteSuccess,g_mapName[g_choosed])
  }
  if(g_choosed != -1) {
    if(is_user_connected(id)) {
      new menuBody[512]
      new len = format(menuBody, 511, g_coloredMenus ? _T("\yThe winner: \w%s^n^n",id) : _T("The winner: %s^n^n",id), g_mapName[g_choosed])
      len += copy(menuBody[len], 511 - len, g_coloredMenus ? _T("\yDo you want to continue?^n\w",id) : _T("Do you want to continue?^n",id))
      copy(menuBody[len], 511 - len, _T("^n1. Yes^n2. No",id))
      show_menu(id, 0x03, menuBody, 10, "The winner: ")
      set_task(10.0, "autoRefuse", 4545454)
    }
    else {
      new modName[8]
      get_modname(modName, 7)
      if(!equal(modName, "zp")) {
        message_begin(MSG_ALL, SVC_INTERMISSION)
        message_end()
      }
      set_task(2.0, "delayedChange", 33119988, g_mapName[g_choosed], strlen(g_mapName[g_choosed]) + 1)
    }
  }
  else {
    client_print(0, print_chat, _T(g_VoteFailed))
    log_to_file(g_logFile,"Vote: %s",g_VoteFailed)
  }
  remove_task(34567 + id)
}

public voteCount(id,key) {
  if(g_hasVoted[id]) {
    return PLUGIN_HANDLED
  }
  g_hasVoted[id] = true
  if(key == 8) {
    return PLUGIN_HANDLED
  }
  if(key == 9) {
    client_print(0, print_chat, _T("Voting has been canceled"))
    remove_task(34567+id)
    remove_task(1467852)
    new menuid, keys
    for(new i = 1; i <= g_maxplayers; i++) {
      g_hasVoted[i] = true
      if(!is_user_connected(i)) continue
      get_user_menu(i, menuid, keys)
      if(menuid == menuid_whichmap || menuid == menuid_changemap) {
        show_menu(i, 0, "NULL", 0, "NULL")
      }
    }
    set_cvar_float("amx_last_voting", get_gametime())
    log_to_file(g_logFile,"Vote: Cancel vote session")
    return PLUGIN_HANDLED
  }
  if(get_cvar_float("amx_vote_answers")) {
    new name[32]
    get_user_name(id, name, 31)
    if(g_yesNoVote)
      client_print(0, print_chat, _T("%s voted %s"), name, key ? _T("against") : _T("for"))
    else
      client_print(0, print_chat, _T("%s voted for %s"), name, g_mapName[g_voteSelected[g_voteCaller][key]])
  }
  ++g_voteCount[key]
  return PLUGIN_HANDLED
}

isMapSelected(id,pos) {
  for(new a = 0; a < g_voteSelectedNum[id]; ++a) {
    if(g_voteSelected[id][a] == pos) {
      return 1
    }
  }
  return 0
}

displayVoteMapsMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512], b = 0, start = pos * 7

  if(start >= g_mapsNum) {
    start = pos = g_menuPosition[id] = 0
  }

  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\yVotemap Menu\R%d/%d^n\w^n",id) : _T("Votemap Menu %d/%d^n^n",id),
    pos+1, (g_mapsNum / 7 + (( g_mapsNum % 7) ? 1 : 0)))

  new end = start + 7, keys = (1<<9)

  if(end > g_mapsNum) {
    end = g_mapsNum
  }

  for(new a = start; a < end; ++a) {
    if(g_voteSelectedNum[id] == 4 || isMapSelected(id, pos * 7 + b)) {
      ++b
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%d. %s^n\w", b, g_mapDesc[a])
      else
        len += format(menuBody[len], 511-len, "#. %s^n", g_mapDesc[a])
    }
    else {
      keys |= (1<<b)
      len += format(menuBody[len], 511-len, "%d. %s^n", ++b, g_mapDesc[a])
    }
  }

  if(g_voteSelectedNum[id]) {
    keys |= (1<<7)
    len += format(menuBody[len], 511-len, _T("^n8. Start Voting^n",id))
  }
  else
    len += format(menuBody[len], 511-len, g_coloredMenus ?
      _T("^n\d8. Start Voting^n\w",id) : _T("^n#. Start Voting^n",id))

  if(end != g_mapsNum) {
    len += format(menuBody[len], 511-len, _T("^n9. More...^n0. %s^n",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    len += format(menuBody[len], 511-len, "^n0. %s^n", pos ? _T("Back",id) : _T("Exit",id))

  len += format(menuBody[len], 511-len, g_voteSelectedNum[id] ?
    (g_coloredMenus ? _T("^n\ySelected Maps:^n\w",id) : _T("^nSelected Maps:^n",id)) : _T("^n^n",id))

  for(new c = 0; c < 4; c++) {
    if(c < g_voteSelectedNum[id])
      len += format(menuBody[len], 511-len, "%s^n", g_mapDesc[g_voteSelected[id][c]])
    else
      len += format(menuBody[len], 511-len, "^n" )
  }

  show_menu(id, keys, menuBody, -1, "Votemap Menu")
}

public cmdVoteMapMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  if(get_cvar_float("amx_last_voting") > get_gametime()) {
    client_print(id, print_chat, _T("There is already one voting..."))
    return PLUGIN_HANDLED
  }

  g_voteSelectedNum[id] = 0

  if(g_mapsNum) {
    displayVoteMapsMenu(id, g_menuPosition[id] = 0)
  }
  else {
    console_print(id, _T("There are no maps in menu"))
    client_print(id, print_chat, _T("There are no maps in menu"))
  }

  return PLUGIN_HANDLED
}

public cmdMapsMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  if(g_mapsNum) {
    displayMapsMenu(id, g_menuPosition[id] = 0)
  }
  else {
    console_print(id, _T("There are no maps in menu"))
    client_print(id, print_chat, _T("There are no maps in menu"))
  }

  return PLUGIN_HANDLED
}

public delayedChange(mapname[]) {
  server_cmd("changelevel %s", mapname)
}

public actionVoteMapMenu(id,key) {
  switch(key) {
    case 7: {
      new Float:voting = get_cvar_float("amx_last_voting")
      if(voting > get_gametime()) {
        client_print(id, print_chat, _T("There is already one voting..."))
        return PLUGIN_HANDLED
      }
      if(voting && voting + get_cvar_float("amx_vote_delay") > get_gametime()) {
        client_print(id, print_chat, _T("Voting not allowed at this time"))
        return PLUGIN_HANDLED
      }

      g_voteCount = {0, 0, 0, 0, 0}

      new Float:vote_time = get_cvar_float("amx_vote_time") + 2.0
      set_cvar_float("amx_last_voting", get_gametime() + vote_time)
      new iVoteTime = floatround(vote_time)

      set_task(vote_time, "checkVotes", 34567 + id)
      g_voteCaller = id
      
      for(new i = 1; i <= g_maxplayers; i++) {
        g_hasVoted[i] = false
      }

      new menuBody[256]
      new players[32]
      new pnum, keys, len
      new menuid, tempkeys
      
      get_players(players, pnum, "c")
      for(new b = 0; b < pnum; ++b) {
        keys = 0
        if(g_voteSelectedNum[id] > 1) {
          len = format(menuBody, 255, g_coloredMenus ?
            _T("\yWhich map do you want?^n\w^n",players[b]) : _T("Which map do you want?^n^n",players[b]))
          for(new c = 0; c < g_voteSelectedNum[id]; ++c) {
            len += format(menuBody[len], 255-len, "%d. %s^n", c + 1, g_mapDesc[g_voteSelected[id][c]])
            keys |= (1<<c)
          }
          keys |= (1<<8)
          len += format(menuBody[len], 255-len, _T("^n9. None^n",players[b]))
        }
        else {
          len = format(menuBody, 255, g_coloredMenus ? _T("\yChange map to^n%s?^n\w^n1. Yes^n2. No^n",players[b])
            : _T("Change map to^n%s?^n^n1. Yes^n2. No^n", players[b]), g_mapDesc[g_voteSelected[id][0]])
          keys = (1<<0)|(1<<1)
        }
        if(players[b] != id) {
          get_user_menu(players[b], menuid, tempkeys)
          if(tempkeys == 0 || menuid < 0) {
            show_menu(players[b], keys, menuBody, iVoteTime, g_voteSelectedNum[id] > 1 ? "Which map do you want?" : "Change map to")
          }
          g_menukeys[players[b]] = keys
          g_menumsg[players[b]][0] = '^0'
          copy(g_menumsg[players[b]], 255, menuBody)
        }
        else {
          format(menuBody[len], 511, _T("^n0. Cancel Vote",id))
          keys |= (1<<9)
          show_menu(id, keys, menuBody, iVoteTime, g_voteSelectedNum[id] > 1 ? "Which map do you want?" : "Change map to")
          g_menukeys[players[b]] = keys
          g_menumsg[players[b]][0] = '^0'
          copy(g_menumsg[players[b]], 255, menuBody)
        }
      }
    
      g_yesNoVote = g_voteSelectedNum[id] > 1 ? 0 : 1
      g_menutitle[0] = '^0'
      copy(g_menutitle, 31, g_voteSelectedNum[id] > 1 ? "Which map do you want?" : "Change map to")
      remove_task(1467852)
      new param[3]
      param[0] = 1
      param[1] = g_voteSelectedNum[id] > 1 ? menuid_whichmap : menuid_changemap
      param[2] = get_cvar_num("amx_vote_time")
      set_task(CHECK_MENU_TIME, "check_menu", 1467852, param, 3)

      new authid[32], name[32]
      get_user_authid(id, authid, 31)
      get_user_name(id, name, 31)

      switch(get_cvar_num("amx_show_activity")) {
        case 2: client_print(0, print_chat, _T("ADMIN %s: vote map(s)"), name)
        case 1: client_print(0, print_chat, _T("ADMIN: vote map(s)"))
      }

      log_to_file(g_logFile,"Vote: ^"%s<%d><%s><>^" vote maps (map#1 ^"%s^") (map#2 ^"%s^") (map#3 ^"%s^") (map#4 ^"%s^")",
        name,get_user_userid(id),authid,
        g_voteSelectedNum[id] > 0 ? g_mapName[g_voteSelected[id][0]] : "",
        g_voteSelectedNum[id] > 1 ? g_mapName[g_voteSelected[id][1]] : "",
        g_voteSelectedNum[id] > 2 ? g_mapName[g_voteSelected[id][2]] : "",
        g_voteSelectedNum[id] > 3 ? g_mapName[g_voteSelected[id][3]] : "")
    }
    case 8: displayVoteMapsMenu(id, ++g_menuPosition[id])
    case 9: displayVoteMapsMenu(id, --g_menuPosition[id])
    default: {
      g_voteSelected[id][g_voteSelectedNum[id]++] = g_menuPosition[id] * 7 + key
      displayVoteMapsMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

public actionMapsMenu(id,key) {
  switch(key) {
    case 8: displayMapsMenu(id, ++g_menuPosition[id])
    case 9: displayMapsMenu(id, --g_menuPosition[id])
    default: {
      new a = g_menuPosition[id] * 8 + key

      new modName[8]
      get_modname(modName, 7)
      if(!equal(modName, "zp")) {
        message_begin(MSG_ALL, SVC_INTERMISSION)
        message_end()
      }

      new authid[32], name[32]
      get_user_authid(id, authid, 31)
      get_user_name(id, name, 31)

      switch(get_cvar_num("amx_show_activity")) {
      case 2: client_print(0, print_chat, _T("ADMIN %s: changelevel %s"), name, g_mapName[a])
      case 1: client_print(0, print_chat, _T("ADMIN: changelevel %s"), g_mapName[a])
      }

      log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" changelevel ^"%s^"",
        name,get_user_userid(id),authid,g_mapName[a])

      set_task(2.0, "delayedChange", 33119955, g_mapName[a], strlen(g_mapName[a]) + 1)
    }
  }
  return PLUGIN_HANDLED
}

displayMapsMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new start = pos * 8
  new b = 0

  if(start >= g_mapsNum)
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\yChangelevel Menu\R%d/%d^n\w^n",id) : _T("Changelevel Menu %d/%d^n^n",id),
    pos+1, (g_mapsNum / 8 + (( g_mapsNum % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)

  if(end > g_mapsNum)
    end = g_mapsNum

  for(new a = start; a < end; ++a) {
    keys |= (1<<b)
    len += format(menuBody[len], 511-len, "%d. %s^n", ++b, g_mapDesc[a])
  }

  if(end != g_mapsNum) {
    format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else format(menuBody[len], 511-len, "^n0. %s", pos ? _T("Back",id) : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Changelevel Menu")
}

loadSettings(filename[]) {
  if(!file_exists(filename))
    return 0

  g_mapsNum = 0

  new text[256], szDesc[48]
  new a, pos = 0

  while(g_mapsNum < MAX_MAPS && (pos = read_file(filename, pos, text, 255, a))) {
    if(!a || text[0] == ';' || text[0] == '/') continue

    parse(text, g_mapName[g_mapsNum], 31, szDesc, 47)

    replace_all(g_mapName[g_mapsNum], 31, " ", "")
    replace_all(g_mapName[g_mapsNum], 31, ".bsp", "")

    if(!is_map_valid(g_mapName[g_mapsNum])) continue

    if(strlen(szDesc) > 31) {
      copy(g_mapDesc[g_mapsNum], 28, szDesc)
      g_mapDesc[g_mapsNum][28] = g_mapDesc[g_mapsNum][29] = g_mapDesc[g_mapsNum][30] = '.'
      g_mapDesc[g_mapsNum][31] = 0
    }
    else {
      if(szDesc[0]) copy(g_mapDesc[g_mapsNum], 31, szDesc)
      else copy(g_mapDesc[g_mapsNum], 31, g_mapName[g_mapsNum])
    }

    g_mapsNum++
  }

  return g_mapsNum
}

loadMapsFolder() {
  g_mapsNum = 0

  new len, pos = 2
  new currentMap[32]
  get_mapname(currentMap, 31)

  while(g_mapsNum < MAX_MAPS && (pos = read_dir("maps", pos, g_mapName[g_mapsNum], 31, len))) {
    if(len <= 4 || (len > 4 && !equali(g_mapName[g_mapsNum][len-4], ".bsp", 4))) continue

    g_mapName[g_mapsNum][len-4] = '^0'
    if(!is_map_valid(g_mapName[g_mapsNum])) continue

    copy(g_mapDesc[g_mapsNum], 31, g_mapName[g_mapsNum])
    g_mapsNum++
  }

  return g_mapsNum
}
