/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Admin commands:
 * amx_kickmenu - displays kick menu
 * amx_banmenu - displays ban menu
 * amx_unbanmenu - displays unban menu
 * amx_slapmenu - displays slap/slay menu
 * amx_teammenu - displays team menu
 * amx_clcmdmenu - displays client commands menu
 *
 */


#include <translator>
#include <amxmod>
#include <amxmisc>
#include <fun>
#include <VexdUM>


new g_menuPosition[33]
new g_menuPlayers[33][32]
new g_menuPlayersNum[33]
new g_menuOption[33]
new g_menuSettings[33]
new g_menuBanSteamid[33]
new g_menuUnBanType[33]
new g_menuUnBanLine[33][2]
new g_menuUnBanText[33][8][32]
new g_bannedCfgFile[2][] = {"banned.cfg","listip.cfg"}

new g_menuSelect[33][64]
new g_menuSelectNum[33]

#define MAX_CLCMDS  24

new g_clcmdName[MAX_CLCMDS][32]
new g_clcmdCmd[MAX_CLCMDS][64]
new g_clcmdMisc[MAX_CLCMDS][2]
new g_clcmdNum

new g_logFile[16]

new g_coloredMenus
new g_cstrikeRunning
new g_dodRunning
new g_funRunning
new g_vexdRunning

stock is_user_really_admin(id) {
  return (get_user_flags(id) & read_flags("acdefghijklmnopqrstu")) ? 1 : 0
}

stock setUserTeam_cs(index,team,model[]="",respawn=0,updatescoreboard=1) {
  if(is_user_connected(index)) {
    if(team >= 0 && team <= 3) {
      set_offset_int(index, 114, team)
      if(is_user_alive(index)) {
        engclient_cmd(index, "drop", "weapon_c4")
        if(team != 2) {
          new defuser = get_offset_int(index, 193)
          if(defuser & (1<<16)) {
            defuser -= (1<<16)
            set_offset_int(index, 193, defuser)
          }
          message_begin(MSG_ONE, get_user_msgid("StatusIcon"), {0,0,0}, index)
          write_byte(0)
          write_string("defuser")
          write_byte(0)
          write_byte(255)
          write_byte(0)
          message_end()
        }
        if(model[0]) {
          set_user_model(index, model)
        }
      }
      if(respawn) {
        new health = get_user_health(index)
        new armor = get_user_armor(index)
        entity_set_int(index, EV_INT_spawnflags, 3)
        DispatchSpawn(index)
        entity_set_int(index, EV_INT_iuser1, 0)
        entity_set_float(index, EV_FL_health, float(health))
        entity_set_float(index, EV_FL_armorvalue, float(armor))
      }
      if(updatescoreboard) {
        message_begin(MSG_ALL, get_user_msgid("TeamInfo"))
        write_byte(index)
        switch(team) {
          case 0: write_string("UNASSIGNED")
          case 1: write_string("TERRORIST")
          case 2: write_string("CT")
          case 3: write_string("SPECTATOR")
        }
        message_end()
      }
      return 1
    }
  }
  return 0
}

public disable_natives_check() {
}

public plugin_init() {
  load_translations("plmenu")
  register_plugin(_T("Players Menu"),AMX_VERSION,"AMX Team")
  register_clcmd("amx_kickmenu","cmdKickMenu",ADMIN_KICK,_T("- displays kick menu"))
  register_clcmd("amx_banmenu","cmdBanMenu",ADMIN_BAN,_T("- displays ban menu"))
  register_clcmd("amx_unbanmenu","cmdUnBanMenu",ADMIN_BAN,_T("- displays unban menu"))
  register_clcmd("amx_slapmenu","cmdSlapMenu",ADMIN_SLAY,_T("- displays slap/slay menu"))
  register_clcmd("amx_teammenu","cmdTeamMenu",ADMIN_LEVEL_A,_T("- displays team menu"))
  register_clcmd("amx_clcmdmenu","cmdClcmdMenu",ADMIN_LEVEL_A,_T("- displays client cmds menu"))

  register_menucmd(register_menuid("Ban Menu"),1023,"actionBanMenu")
  register_menucmd(register_menuid("UnBan STEAMID or IP?"),(1<<0|1<<1|1<<9),"actionUnBanMenuType")
  register_menucmd(register_menuid("UnBan  Menu"),1023,"actionUnBanMenu")
  register_menucmd(register_menuid("Kick Menu"),1023,"actionKickMenu")
  register_menucmd(register_menuid("Slap/Slay Menu"),1023,"actionSlapMenu")
  register_menucmd(register_menuid("Team Menu"),1023,"actionTeamMenu")
  register_menucmd(register_menuid("Client Cmds Menu"),1023,"actionClcmdMenu")

  g_coloredMenus = colored_menus()
  g_cstrikeRunning = (is_running("cstrike") || is_running("czero"))
  g_dodRunning = (is_running("dod"))
  g_funRunning = (is_module_running("Fun"))
  g_vexdRunning = (is_module_running("VexdUM"))

  new filename[64]
  build_path(filename, 63, "$configdir/clcmds.ini")
  load_settings(filename)

  get_logfile(g_logFile, 15)
}

/* Ban menu */

public actionBanMenu(id,key) {
  switch(key) {
    case 6: {
      g_menuBanSteamid[id] = !g_menuBanSteamid[id]
      displayBanMenu(id, g_menuPosition[id])
    }
    case 7: {
      ++g_menuOption[id]
      g_menuOption[id] %= 5

      switch(g_menuOption[id]) {
        case 0: g_menuSettings[id] = 0
        case 1: g_menuSettings[id] = 5
        case 2: g_menuSettings[id] = 15
        case 3: g_menuSettings[id] = 30
        case 4: g_menuSettings[id] = 60
      }

      displayBanMenu(id, g_menuPosition[id])
    }
    case 8: displayBanMenu(id, ++g_menuPosition[id])
    case 9: displayBanMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 6 + key]

      new name[32], name2[32], authid[32], authid2[32]
      get_user_name(player, name2, 31)
      get_user_authid(id, authid, 31)
      get_user_authid(player, authid2, 31)
      get_user_name(id, name, 31)
      new userid2 = get_user_userid(player)

      log_to_file(g_logFile,"Ban: ^"%s<%d><%s><>^" ban and kick ^"%s<%d><%s><>^" (minutes ^"%d^")",
        name,get_user_userid(id),authid,name2,userid2,authid2,g_menuSettings[id])

      switch(get_cvar_num("amx_show_activity")) {
        case 2: client_print(0, print_chat, _T("ADMIN %s: ban %s"), name, name2)
        case 1: client_print(0, print_chat, _T("ADMIN: ban %s"), name2)
      }

      if(equal("4294967295",authid2)
      || equal("HLTV",authid2)
      || equal("STEAM_ID_LAN",authid2)
      || equali("VALVE_ID_LAN", authid2)
      || g_menuBanSteamid[id] == 0) {
        new ipa[32]
        get_user_ip(player, ipa, 31, 1)
        server_cmd("addip %d %s;writeip", g_menuSettings[id], ipa)
      }
      else
        server_cmd("banid %d #%d kick;writeid", g_menuSettings[id], userid2)

      server_exec()

      displayBanMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displayBanMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 6

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\yBan Menu\R%d/%d^n\w^n",id) :
    _T("Ban Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 6 + ((g_menuPlayersNum[id] % 6) ? 1 : 0)))

  new end = start + 6
  new keys = (1<<9)|(1<<7)|(1<<6)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if(is_user_bot(i) || (get_user_flags(i) & ADMIN_IMMUNITY)) {
      ++b
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += format(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_really_admin(i) ? "\r" : "", name)
      else
        len += format(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(g_menuBanSteamid[id])
    len += format(menuBody[len], 511-len, _T("^n7. Ban by STEAMID^n",id))
  else
    len += format(menuBody[len], 511-len, _T("^n7. Ban by IP^n",id))
    
  if(g_menuSettings[id])
    len += format(menuBody[len], 511-len, _T("8. Ban for %d minutes^n",id), g_menuSettings[id])
  else
    len += format(menuBody[len], 511-len, _T("8. Ban permanently^n",id))

  if(end != g_menuPlayersNum[id]) {
    format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    format(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Ban Menu")
}

public cmdBanMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  g_menuOption[id] = 1
  g_menuSettings[id] = 5
  g_menuBanSteamid[id] = 1
  displayBanMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

/* UnBan menu */

public actionUnBanMenu(id,key) {
  switch(key) {
    case 8: {
      displayUnBanMenu(id, ++g_menuPosition[id])
    }
    case 9: {
      g_menuUnBanLine[id][0] = g_menuUnBanLine[id][0] - (8 + g_menuUnBanLine[id][1])
      if(g_menuUnBanLine[id][0] < 0) g_menuUnBanLine[id][0] = 0
      displayUnBanMenu(id, --g_menuPosition[id])
    }
    default: {
      new name[32], authid[32]
      get_user_authid(id, authid, 31)
      get_user_name(id, name, 31)

      log_to_file(g_logFile,"UnBan: ^"%s<%d><%s><>^" unban ^"%s^"",
        name,get_user_userid(id),authid,g_menuUnBanText[id][key])

      switch(get_cvar_num("amx_show_activity")) {
        case 2: client_print(0, print_chat, _T("ADMIN %s: unban %s"), name, g_menuUnBanText[id][key])
        case 1: client_print(0, print_chat, _T("ADMIN: unban %s"), g_menuUnBanText[id][key])
      }

      if(g_menuUnBanType[id] == 1) {
        server_cmd("removeip ^"%s^"; writeip", g_menuUnBanText[id][key])
        server_exec()
        console_print(id, _T("IP ^"%s^" removed from ban list"), g_menuUnBanText[id][key])
      }
      else {
        server_cmd("removeid %s; writeid",  g_menuUnBanText[id][key])
        console_print(id, _T("Authid ^"%s^" removed from ban list"),  g_menuUnBanText[id][key])
      }

      g_menuUnBanLine[id][0] = g_menuUnBanLine[id][1] = 0
      displayUnBanMenu(id, g_menuPosition[id] = 0)
    }
  }
  return PLUGIN_HANDLED
}

checkSTEAMID(steamid[]) {
  new len = strlen(steamid)
  if(len > 10 && equali(steamid, "STEAM_", 6) && steamid[7] == ':' && steamid[9] == ':' && str_to_num(steamid[10])) {
    return 1
  }
  return 0
}

checkIP(ip[]) {
  new len = strlen(ip)
  new dots = 0, i = 0
  while(isdigit(ip[i]) || ip[i]=='.')
    if(ip[i++] == '.')
      dots++
  if(i == len && dots == 3) {
    return 1
  }
  return 0
}

displayUnBanMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\yUnBan Menu\R%d^n\w^n",id) :
    _T("UnBan Menu %d^n^n",id),pos+1)

  new keys = (1<<9)
  new textlen, line
  new type = g_menuUnBanType[id]
  new temp[32], banTime[32], disable

  if(file_exists(g_bannedCfgFile[type])) {
    line = g_menuUnBanLine[id][0]
    while((line = read_file(g_bannedCfgFile[type], line, g_menuSelect[id], 63, textlen))) {
      temp[0] = '^0'
      banTime[0] = '^0'
      g_menuUnBanText[id][b][0] = '^0'
      disable = 0
      if(textlen < 9)
        disable = 1
      else if(parse(g_menuSelect[id], temp, 31, banTime, 31, g_menuUnBanText[id][b], 31) != 3)
        disable = 1
      else if((type == 1 && !checkIP(g_menuUnBanText[id][b])) || (type == 0 && !checkSTEAMID(g_menuUnBanText[id][b])))
        disable = 1
      if(disable == 0) {
        keys |= (1<<b)
        if(g_coloredMenus)
          len += format(menuBody[len], 511-len, "%d. %s\R\r%s^n\w", b, g_menuUnBanText[id][b++], banTime)
        else
          len += format(menuBody[len], 511-len, "%d. %s   ( %s )^n", b, g_menuUnBanText[id][b++], banTime)
      }
      else {
        if(g_coloredMenus)
          len += format(menuBody[len], 511-len, "\d%d. %s^n\w", b, g_menuUnBanText[id][b++])
        else
          len += format(menuBody[len], 511-len, "#. %s^n", b, g_menuUnBanText[id][b++])
      }
      if(b == 8) break
    }

    if(b == 8 && read_file(g_bannedCfgFile[type], line, g_menuSelect[id], 63, textlen) > 0) {
      format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id),
        pos ? _T("Back",id)
            : _T("Exit",id))
      keys |= (1<<8)
    }
    else
      format(menuBody[len], 511-len, "^n0. %s",
        pos ? _T("Back",id)
            : _T("Exit",id))

    g_menuUnBanLine[id][1] = line - g_menuUnBanLine[id][0]
    g_menuUnBanLine[id][0] = line
    show_menu(id, keys, menuBody, -1, "UnBan  Menu")
  }

  return
}

public actionUnBanMenuType(id,key) {
  switch(key) {
    case 9: return PLUGIN_HANDLED
    default: {
      g_menuUnBanType[id] = key // 0 = STEAMID, 1 = IP
      g_menuUnBanLine[id][0] = g_menuUnBanLine[id][1] = 0
      displayUnBanMenu(id, g_menuPosition[id] = 0)
    }
  }
  return PLUGIN_HANDLED
}

displayUnBanMenuType(id) {
  new menuBody[512]
  
  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\yUnBan STEAMID or IP?^n\w^n",id) :
    _T("UnBan STEAMID or IP?^n^n",id))

  new keys = (1<<0)|(1<<1)|(1<<9)

  len += format(menuBody[len], 511-len, _T("1. STEAMID^n",id))
  len += format(menuBody[len], 511-len, _T("2. IP^n",id))
  format(menuBody[len], 511-len, _T("^n0. Exit",id))

  show_menu(id, keys, menuBody, -1, "UnBan STEAMID or IP?")
}

public cmdUnBanMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  g_menuUnBanType[id] = -1
  displayUnBanMenuType(id)
  return PLUGIN_HANDLED
}

/* Slap/Slay */

public actionSlapMenu(id,key) {
  switch(key) {
    case 7: {
      ++g_menuOption[id]
      g_menuOption[id] %= 4
      switch(g_menuOption[id]) {
        case 1: g_menuSettings[id] = 0
        case 2: g_menuSettings[id] = 1
        case 3: g_menuSettings[id] = 5
      }
      displaySlapMenu(id, g_menuPosition[id])
    }
    case 8: displaySlapMenu(id, ++g_menuPosition[id])
    case 9: displaySlapMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 7 + key]

      new name2[32]
      get_user_name(player, name2, 31)

      if(!is_user_alive(player)) {
        client_print(id, print_chat, _T("That action can't be performed on dead client ^"%s^""), name2)
        displaySlapMenu(id, g_menuPosition[id])
        return PLUGIN_HANDLED
      }

      new authid[32], authid2[32], name[32]

      get_user_authid(id, authid, 31)
      get_user_authid(player, authid2, 31)
      get_user_name(id, name, 31)

      if(g_menuOption[id]) {
        log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" slap with %d damage ^"%s<%d><%s><>^"",
          name,get_user_userid(id),authid, g_menuSettings[id],name2,get_user_userid(player),authid2)
        switch(get_cvar_num("amx_show_activity")) {
          case 2: client_print(0, print_chat, _T("ADMIN %s: slap %s with %d damage"), name, name2, g_menuSettings[id])
          case 1: client_print(0, print_chat, _T("ADMIN: slap %s with %d damage"), name2, g_menuSettings[id])
        }
      }
      else {
        log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" slay ^"%s<%d><%s><>^"",
          name,get_user_userid(id),authid,name2,get_user_userid(player),authid2)
        switch(get_cvar_num("amx_show_activity")) {
          case 2: client_print(0, print_chat, _T("ADMIN %s: slay %s"), name, name2)
          case 1: client_print(0, print_chat, _T("ADMIN: slay %s"), name2)
        }
      }

      if(g_menuOption[id])
        user_slap(player, (get_user_health(player) >  g_menuSettings[id]) ? g_menuSettings[id] : 0)
      else
        user_kill(player)

      displaySlapMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displaySlapMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 7

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\ySlap/Slay Menu\R%d/%d^n\w^n",id) :
    _T("Slap/Slay Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 7 + ((g_menuPlayersNum[id] % 7) ? 1 : 0)))

  new end = start + 7
  new keys = (1<<9)|(1<<7)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if(!is_user_alive(i) || ((get_user_flags(i) & ADMIN_IMMUNITY) && i != id)) {
      ++b
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += format(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_really_admin(i) ? "\r" : "", name)
      else
        len += format(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(g_menuOption[id])
    len += format(menuBody[len], 511-len, _T("^n8. Slap with %d damage^n",id), g_menuSettings[id])
  else
    len += format(menuBody[len], 511-len, _T("^n8. Slay^n",id))

  if(end != g_menuPlayersNum[id]) {
    format(menuBody[len], 511-len,_T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    format(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Slap/Slay Menu")
}

public cmdSlapMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  g_menuOption[id] = 1
  g_menuSettings[id] = 0

  displaySlapMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

/* Kick */

public actionKickMenu(id,key) {
  switch(key) {
    case 8: displayKickMenu(id, ++g_menuPosition[id])
    case 9: displayKickMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 8 + key]

      new authid[32], authid2[32], name[32], name2[32]
      get_user_authid(id, authid, 31)
      get_user_authid(player, authid2, 31)
      get_user_name(id, name, 31)
      get_user_name(player, name2, 31)
      new userid2 = get_user_userid(player)

      log_to_file(g_logFile,"Kick: ^"%s<%d><%s><>^" kick ^"%s<%d><%s><>^"",
          name,get_user_userid(id),authid,name2,userid2,authid2)

      switch(get_cvar_num("amx_show_activity")) {
        case 2: client_print(0, print_chat, _T("ADMIN %s: kick %s"), name, name2)
        case 1: client_print(0, print_chat, _T("ADMIN: kick %s"), name2)
      }

      server_cmd("kick #%d", userid2)
      server_exec()

      displayKickMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displayKickMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 8

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\yKick Menu\R%d/%d^n\w^n",id) :
    _T("Kick Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 8 + ((g_menuPlayersNum[id] % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if(get_user_flags(i) & ADMIN_IMMUNITY) {
      ++b
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += format(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_really_admin(i) ? "\r" : "", name)
      else
        len += format(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(end != g_menuPlayersNum[id]) {
    format(menuBody[len], 511-len,_T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    format(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Kick Menu")
}

public cmdKickMenu(id,level,cid) {
  if(cmd_access(id,level,cid,1))
    displayKickMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

/* Team menu */

public actionTeamMenu(id,key) {
  switch(key) {
    case 7: {
      g_menuOption[id]++
      if(g_menuOption[id] == 3 && g_cstrikeRunning && get_cvar_num("allow_spectators") == 0) g_menuOption[id] = 1
      if(g_menuOption[id] == 4) g_menuOption[id] = 1
      displayTeamMenu(id, g_menuPosition[id])
    }
    case 8: displayTeamMenu(id, ++g_menuPosition[id])
    case 9: displayTeamMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 7 + key]
      new authid[32], authid2[32], name[32], name2[32]
      get_user_name(player, name2, 31)
      get_user_authid(id, authid, 31)
      get_user_authid(player, authid2, 31)
      get_user_name(id, name, 31)

      switch(g_menuOption[id]) {
        case 1: {
          if(g_cstrikeRunning) {
            log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" transfer ^"%s<%d><%s><>^" (to ^"%s^")",
              name,get_user_userid(id),authid,name2,get_user_userid(player),authid2,"TERRORISTS")
            switch(get_cvar_num("amx_show_activity")) {
              case 2: client_print(0, print_chat, _T("ADMIN %s: transfer %s to %s"), name, name2, _T("TERRORISTS"))
              case 1: client_print(0, print_chat, _T("ADMIN: transfer %s to %s"), name2, _T("TERRORISTS"))
            }
          }
          else if(g_dodRunning) {
            log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" transfer ^"%s<%d><%s><>^" (to ^"%s^")",
              name,get_user_userid(id),authid,name2,get_user_userid(player),authid2,"ALLIES")
            switch(get_cvar_num("amx_show_activity")) {
              case 2: client_print(0, print_chat, _T("ADMIN %s: transfer %s to %s"), name, name2, _T("ALLIES"))
              case 1: client_print(0, print_chat, _T("ADMIN: transfer %s to %s"), name2, _T("ALLIES"))
            }
          }
          else {
            log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" transfer ^"%s<%d><%s><>^" (to ^"%s^")",
              name,get_user_userid(id),authid,name2,get_user_userid(player),authid2,"TEAM 1")
            switch(get_cvar_num("amx_show_activity")) {
              case 2: client_print(0, print_chat, _T("ADMIN %s: transfer %s to %s"), name, name2, _T("TEAM 1"))
              case 1: client_print(0, print_chat, _T("ADMIN: transfer %s to %s"), name2, _T("TEAM 1"))
            }
          }
        }
        case 2: {
          if(g_cstrikeRunning) {
            log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" transfer ^"%s<%d><%s><>^" (to ^"%s^")",
              name,get_user_userid(id),authid,name2,get_user_userid(player),authid2,"CTs")
            switch(get_cvar_num("amx_show_activity")) {
              case 2: client_print(0, print_chat, _T("ADMIN %s: transfer %s to %s"), name, name2, _T("CTs"))
              case 1: client_print(0, print_chat, _T("ADMIN: transfer %s to %s"), name2, _T("CTs"))
            }
          }
          else if(g_dodRunning) {
            log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" transfer ^"%s<%d><%s><>^" (to ^"%s^")",
              name,get_user_userid(id),authid,name2,get_user_userid(player),authid2,"AXIS")
            switch(get_cvar_num("amx_show_activity")) {
              case 2: client_print(0, print_chat, _T("ADMIN %s: transfer %s to %s"), name, name2, _T("AXIS"))
              case 1: client_print(0, print_chat, _T("ADMIN: transfer %s to %s"), name2, _T("AXIS"))
            }
          }
          else {
            log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" transfer ^"%s<%d><%s><>^" (to ^"%s^")",
              name,get_user_userid(id),authid,name2,get_user_userid(player),authid2,"TEAM 2")
            switch(get_cvar_num("amx_show_activity")) {
              case 2: client_print(0, print_chat, _T("ADMIN %s: transfer %s to %s"), name, name2, _T("TEAM 2"))
              case 1: client_print(0, print_chat, _T("ADMIN: transfer %s to %s"), name2, _T("TEAM 2"))
            }
          }
        }
        case 3: {
          log_to_file(g_logFile,"Cmd: ^"%s<%d><%s><>^" transfer ^"%s<%d><%s><>^" (to ^"%s^")",
            name,get_user_userid(id),authid,name2,get_user_userid(player),authid2,"SPECTATORS")
          switch(get_cvar_num("amx_show_activity")) {
            case 2: client_print(0, print_chat, _T("ADMIN %s: transfer %s to %s"), name, name2, _T("SPECTATORS"))
            case 1: client_print(0, print_chat, _T("ADMIN: transfer %s to %s"), name2, _T("SPECTATORS"))
          }
        }
      }

      if(!(g_cstrikeRunning && g_funRunning)) {
        if(!(g_cstrikeRunning && g_vexdRunning)) {
          new limitt
          if(g_cstrikeRunning) {
            limitt = get_cvar_num("mp_limitteams")
            set_cvar_num("mp_limitteams", 0)
          }
          user_kill(player, 1)
          new szTeam[2]
          if(g_cstrikeRunning && g_menuOption[id] == 3) {
            num_to_str(6, szTeam, 1)
          }
          else {
            num_to_str(g_menuOption[id], szTeam, 1)
          }
          engclient_cmd(player, "jointeam", szTeam)
          if(g_menuOption[id] != 3) {
            if(g_cstrikeRunning) {
              engclient_cmd(player, "joinclass", "6")
            }
          }
          //show_menu(player, 0, "NULL", 0, "NULL")
          if(g_cstrikeRunning) {
            client_cmd(player, "slot5")
            set_cvar_num("mp_limitteams", limitt)
          }
        }
        else {
          setUserTeam_cs(player, g_menuOption[id], "", (is_user_alive(player) && g_menuOption[id] != 3) ? 1 : 0, 1)
          if(g_menuOption[id] == 3) user_kill(player, 1)
        }
      }
      else {
        set_user_team_cs(player, g_menuOption[id], "", (is_user_alive(player) && g_menuOption[id] != 3) ? 1 : 0, 1)
        if(g_menuOption[id] == 3) user_kill(player, 1)
      }

      if(player != id)
        displayTeamMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displayTeamMenu(id, pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i, iteam
  new name[32], team[7]
  new start = pos * 7

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\yTeam Menu\R%d/%d^n\w^n",id) :
    _T("Team Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 7 + ((g_menuPlayersNum[id] % 7) ? 1 : 0)))

  new end = start + 7
  new keys = (1<<9)|(1<<7)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)
    
    if((g_cstrikeRunning && g_funRunning)) {
      iteam = get_user_team_cs(i)
      switch(iteam) {
        case 1: copy(team, 6, "TERROR")
        case 2: copy(team, 6, "CT")
        case 3: copy(team, 6, "SPECTA")
      }
    }
    else if((g_cstrikeRunning && g_vexdRunning)) {
      iteam = get_offset_int(i, 114)
      switch(iteam) {
        case 1: copy(team, 6, "TERROR")
        case 2: copy(team, 6, "CT")
        case 3: copy(team, 6, "SPECTA")
      }
    }
    else {
      iteam = get_user_team(i, team, 6)
      if(equali(team, "TERROR", 6) || equali(team, "Allies", 6)) iteam = 1
      if(equali(team, "CT", 2) || equali(team, "Axis", 4)) iteam = 2
      if(equali(team, "SPECTA", 6)) iteam = 3
    }
  
    if((iteam == g_menuOption[id]) || ((get_user_flags(i) & ADMIN_IMMUNITY) && i != id)) {
      ++b
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%d. %s\R%s^n\w", b, name, team)
      else
        len += format(menuBody[len], 511-len, "#. %s   %s^n", name, team)
    }
    else {
      keys |= (1<<b)
      len += format(menuBody[len], 511-len, g_coloredMenus ?
        "%d. %s\y\R%s^n\w" : "%d. %s   %s^n", ++b, name, team)
    }
  }

  switch(g_menuOption[id]) {
    case 1: {
      if(g_cstrikeRunning) {
        len += format(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("TERRORISTS",id))
      }
      else if(g_dodRunning) {
        len += format(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("ALLIES",id))
      }
      else {
        len += format(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("TEAM 1",id))
      }
    }
    case 2: {
      if(g_cstrikeRunning) {
        len += format(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("CTs",id))
      }
      else if(g_dodRunning) {
        len += format(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("AXIS",id))
      }
      else {
        len += format(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("TEAM 2",id))
      }
    }
    case 3: {
      len += format(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("SPECTATORS",id))
    }
  }

  if(end != g_menuPlayersNum[id]) {
    format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    format(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Team Menu")
}

public cmdTeamMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  g_menuOption[id] = 1

  displayTeamMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

/* Client cmds menu */

public actionClcmdMenu(id,key) {
  switch(key) {
    case 7: {
      ++g_menuOption[id]
      g_menuOption[id] %= g_menuSelectNum[id]
      displayClcmdMenu(id, g_menuPosition[id])
    }
    case 8: displayClcmdMenu(id, ++g_menuPosition[id])
    case 9: displayClcmdMenu(id, --g_menuPosition[id])
    default:{
      new player = g_menuPlayers[id][g_menuPosition[id] * 7 + key]
      new flags = g_clcmdMisc[g_menuSelect[id][g_menuOption[id]]][1]
      if(is_user_connected(player)) {
        new command[64], authid[32], name[32], userid[32]
        copy(command, 63, g_clcmdCmd[g_menuSelect[id][g_menuOption[id]]])
        get_user_authid(player, authid, 31)
        get_user_name(player, name, 31)
        num_to_str(get_user_userid(player), userid, 31)
        replace(command, 63, "%userid%", userid)
        replace(command, 63, "%authid%", authid)
        replace(command, 63, "%name%", name)
        if(flags & 1) {
          server_cmd(command)
          server_exec()
        }
        else if(flags & 2) {
          new cmd[32], cmdaccess[32], playerflags, cmdflags, noaccess = 0
          playerflags = get_user_flags(id)
          parse(command, cmd, 31)
          if(get_cmdaccess(cmd, cmdaccess, 31)) {
            cmdflags = read_flags(cmdaccess)
            if((playerflags & cmdflags) != cmdflags) noaccess = 1
            if(noaccess) {
              set_user_flags(id, cmdflags)
            }
            client_cmd(id, command)
            if(noaccess) {
              new param[3]
              param[0] = id
              param[1] = playerflags
              set_task(0.3, "setOldFlags", 400+cmdflags+id, param, 3)
            }
          }
          else client_cmd(id, command)
        }
        else if(flags & 4)
          client_cmd(player, command)
      }
      if(flags & 8) displayClcmdMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

public setOldFlags(param[]) {
  new id = param[0]
  new playerflags = param[1]
  if(is_user_connected(id)) {
    remove_user_flags(id)
    set_user_flags(id, playerflags)
  }
}

displayClcmdMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 7

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    _T("\yClient Cmds Menu\R%d/%d^n\w^n",id) :
    _T("Client Cmds Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 7 + ((g_menuPlayersNum[id] % 7) ? 1 : 0)))

  new end = start + 7
  new keys = (1<<9)|(1<<7)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if(!g_menuSelectNum[id] || ((get_user_flags(i) & ADMIN_IMMUNITY) && i != id)) {
      ++b
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += format(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_really_admin(i) ? "\r" : "", name)
      else
        len += format(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(g_menuSelectNum[id])
    len += format(menuBody[len], 511-len, "^n8. %s^n", is_translated(g_clcmdName[g_menuSelect[id][g_menuOption[id]]]) ? _T(g_clcmdName[g_menuSelect[id][g_menuOption[id]]],id) : g_clcmdName[g_menuSelect[id][g_menuOption[id]]])
  else
    len += format(menuBody[len], 511-len, _T("^n8. No cmds available^n",id))

  if(end != g_menuPlayersNum[id]) {
    format(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    format(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Client Cmds Menu")
}

public cmdClcmdMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  new flags = get_user_flags(id)

  g_menuSelectNum[id] = 0

  for(new a = 0; a < g_clcmdNum; ++a)
    if((g_clcmdMisc[a][0] & flags) == g_clcmdMisc[a][0])
      g_menuSelect[id][g_menuSelectNum[id]++] = a

  g_menuOption[id] = 0

  displayClcmdMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

load_settings(szFilename[]) {
  if(!file_exists(szFilename))
    return 0

  g_clcmdNum = 0

  new text[256], szFlags[32], szAccess[32]
  new a, pos = 0

  while(g_clcmdNum < MAX_CLCMDS && (pos = read_file(szFilename, pos, text, 255, a))) {
    if(!a || text[0] == ';' || text[0] == '/') continue

    if(parse(text, g_clcmdName[g_clcmdNum], 31,
      g_clcmdCmd[g_clcmdNum], 63, szFlags, 31, szAccess, 31) > 3) {
      replace_all(g_clcmdCmd[g_clcmdNum], 63, "\'", "^"")

      g_clcmdMisc[g_clcmdNum][1] = read_flags(szFlags)
      g_clcmdMisc[g_clcmdNum][0] = read_flags(szAccess)
      g_clcmdNum++
    }
  }

  return 1
}
