/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 *
 * (c) 2006, KRoT@L
 *
 * Modifies amx/config/plugins.ini and amx/config/modules.ini
 * according to the contents of amx/plugins and amx/modules folders.
 *
 */

#include <translator>
#include <amxmod>
#include <amxmisc>

new g_logFile[16]
new g_inLine[128]
new g_pos, g_len

public plugin_init() {
  load_translations("plugmod_manager")
  register_plugin(_T("Plugins-Modules Manager"),AMX_VERSION,"AMX Team")
  register_cvar("amx_manage_plugins","0")
  register_cvar("amx_manage_modules","0")
  get_logfile(g_logFile, 15)
}

in_dir(file[],dir[]) {
  g_pos = 2
  g_len = 0
  while((g_pos = read_dir(dir, g_pos, g_inLine, 127, g_len))) {
    if(contain(file, g_inLine) != -1) return 1
  }
  return 0
}

in_file(text[],file[]) {
  if(!file_exists(file)) return 0
  g_pos = 0
  g_len = 0
  while((g_pos = read_file(file, g_pos, g_inLine, 127, g_len))) {
    if(contain(g_inLine, text) != -1) return 1
  }
  return 0
}

public plugin_end() {
  if(get_cvar_num("amx_manage_plugins")) {
    new filename[128], dir[128]
    get_localinfo("amx_plugins", filename, 127)
    if(!filename[0]) build_path(filename, 127, "$configdir/plugins.ini")
    get_localinfo("amx_pluginsdir", dir, 127)
    if(!dir[0]) build_path(dir, 127, "$basedir/plugins")
    new plugin[128], pos = 2, len
    while((pos = read_dir(dir, pos, plugin, 127, len))) {
      if(contain(plugin, ".amx") != -1 && !in_file(plugin, filename)) {
        write_file(filename, plugin)
      }
    }
    pos = 0
    while((pos = read_file(filename, pos, plugin, 127, len))) {
      if(plugin[0] && contain(plugin, ".amx") != -1 && contain(plugin,"pluginname.amx") == -1 && !in_dir(plugin, dir)) {
        write_file(filename, "", pos-1)
      }
    }
    log_to_file(g_logFile,"[AMX] Plugins-Modules Manager: File %s modified.",filename)
  }
  if(get_cvar_num("amx_manage_modules")) {
    new filename[128], dir[128]
    new is_linux = is_linux_server()
    get_localinfo("amx_modules", filename, 127)
    if(!filename[0]) build_path(filename, 127, "$configdir/modules.ini")
    get_localinfo("amx_modulesdir", dir, 127)
    if(!dir[0]) build_path(dir, 127, "$basedir/modules")
    new module[128], pos = 0, len = 0
    while((pos = read_dir(dir, pos, module, 127, len))) {
      if(!in_file(module, filename)) {
        if(len > 3 && is_linux && equal(module[len-3], ".so", 3)
        || len > 4 && !is_linux && equal(module[len-4], ".dll", 4)) {
          write_file(filename, module)
        }
      }
    }
    pos = 0
    while((pos = read_file(filename, pos, module, 127, len))) {
      if((len > 3 && equal(module[len-3], ".so", 3)) &&
      (!in_dir(module, dir) || !is_linux) || (len > 4 && equal(module[len-4], ".dll", 4)) &&
      (!in_dir(module, dir) || is_linux)) {
        write_file(filename, "", pos-1)
      }
    }
    log_to_file(g_logFile,"[AMX] Plugins-Modules Manager: File %s modified.",filename)
  }
}
