/*
 * Copyright (c) 2002-2006 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 *
 * Commands:
 * say timeleft - displays timeleft (available for all clients) 
 * amx_time_display < flags time > ... - sets time displaying
 * Flags:
 * "a" - display text
 * "b" - use voice
 * "c" - don't add "remaining" (only in voice)
 * "d" - don't add "hours/minutes/seconds" (only in voice)
 * "e" - show/speak if current time is less than this set
 * Example:
 * amx_time_display "ab 600" "ab 300" "ab 180" "ab 60" "bcde 11"
 *
 * Cvars:
 * amx_time_voice < 1|0 > - announces "say thetime"
 * and "say timeleft"  with voice when set to 1
 *
 */

#include <translator>
#include <amxmod>

new g_TimeSet[32][2]
new g_LastTime
new g_CountDown
new g_Switch
new g_nextMap[32]
new Float:g_gameTime
new g_TimeLeftInit

public plugin_init() {
  load_translations("timeleft")
  register_plugin(_T("TimeLeft"),AMX_VERSION,"AMX Team")
  register_cvar("amx_time_voice","0")
  register_srvcmd("amx_time_display","setDisplaying")
  register_cvar("amx_timeleft","00:00",FCVAR_SERVER|FCVAR_EXTDLL|FCVAR_UNLOGGED|FCVAR_SPONLY)
  register_clcmd("say timeleft","sayTimeLeft",0,_T("- displays timeleft"))
  register_clcmd("say thetime","sayTheTime",0,_T("- displays current time"))
  g_TimeLeftInit = get_timeleft()
  set_task(0.1, "checkTimeLeftInit", 34976182)
}

public plugin_cfg() {
  get_cvar_string("amx_nextmap", g_nextMap, 31)
}

public checkTimeLeftInit() {
  new timeleft = get_timeleft()
  if(timeleft == g_TimeLeftInit) {
    set_task(0.1, "checkTimeLeftInit", 34976182)
  }
  else {
    set_task(1.0, "timeRemain", 8648458)
    g_gameTime = get_gametime()
  }
}

public sayTheTime(id) {
  if(get_cvar_num("amx_time_voice")) {
    new mhours[6], mmins[6], whours[32], wmins[32], wpm[6]
    get_time("%H", mhours, 5)
    get_time("%M", mmins, 5)
    new mins = str_to_num(mmins)
    new hrs = str_to_num(mhours)
    if(mins)
      num_to_word(mins, wmins, 31)
    else
      wmins[0] = '^0'
    if(hrs < 12)
      wpm = "am "
    else {
      if(hrs > 12) hrs -= 12
      wpm = "pm "
    }
    if(hrs)
      num_to_word(hrs, whours, 31)
    else
      whours = "twelve "
    client_cmd(id, "spk ^"fvox/time_is_now %s_period %s%s^"", whours, wmins, wpm)
  }
  new ctime[64]
  get_time("%m/%d/%Y - %H:%M:%S", ctime, 63)
  client_print(0, print_chat, _T("The time:   %s"), ctime)
  return PLUGIN_CONTINUE
}

public sayTimeLeft(id) {
  if(get_cvar_float("mp_timelimit")) {
    new a = get_timeleft()
    if(get_cvar_num("amx_time_voice")) {
      new svoice[128]
      setTimeVoice(svoice, 127, 0, a)
      client_cmd(id, svoice)
    }
    client_print(0, print_chat, _T("Time Left:  %d:%02d"), (a / 60), (a % 60))
  }
  else
    client_print(0, print_chat, _T("No Time Limit"))
  return PLUGIN_CONTINUE
}

setTimeText(text[],len,tmlf,dest) {
  new secs = tmlf % 60
  new mins = tmlf / 60
  if(secs == 0)
    format(text, len, "%d %s", mins, (mins > 1) ? _T("minutes",dest) : _T("minute",dest))
  else if(mins == 0)
    format(text, len, "%d %s", secs, (secs > 1) ? _T("seconds",dest) : _T("second",dest))
  else
    format(text, len, "%d %s %d %s", mins, (mins > 1) ? _T("minutes",dest) : _T("minute",dest), secs, (secs > 1) ? _T("seconds",dest) : _T("second",dest))
}

setTimeVoice(text[],len,flags,tmlf) {
  new temp[7][32]
  new secs = tmlf % 60
  new mins = tmlf / 60
  for(new a = 0; a < 7; ++a)
    temp[a][0] = '^0'
  if(secs > 0) {
    num_to_word(secs, temp[4], 31)
    if(!(flags & 8)) temp[5] = "seconds " /* there is no "second" in default hl */
  }
  if(mins > 59) {
    new hours = mins / 60
    num_to_word(hours, temp[0], 31)
    if(!(flags & 8)) temp[1] = "hours "
    mins = mins % 60
  }
  if(mins > 0) {
    num_to_word(mins, temp[2], 31)
    if(!(flags & 8)) temp[3] =  "minutes "
  }
  if(!(flags & 4)) temp[6] = "remaining "
  return format(text, len, "spk ^"vox/%s%s%s%s%s%s%s^"", temp[0], temp[1], temp[2], temp[3], temp[4], temp[5], temp[6])
}

findDispFormat(time) {
  for(new i = 0; g_TimeSet[i][0]; ++i) {
    if(g_TimeSet[i][1] & 16) {
      if(g_TimeSet[i][0] > time) {
        if(!g_Switch) {
          g_CountDown = g_Switch = time
          remove_task(8648458)
          new Float:gameTime = get_gametime()
          new Float:offset = gameTime - g_gameTime
          offset = offset - floatround(offset, floatround_floor)
          offset = floatround(offset*10.0, floatround_floor) / 10.0
          set_task(1.0 - offset, "timeRemain", 34543)
        }
        return i
      }
    }
    else if(g_TimeSet[i][0] == time) {
      return i
    }
  }
  return -1
}

public setDisplaying() {
  new arg[32], flags[32], num[32]
  new argc = read_argc() - 1
  new i = 0
  while(i < argc && i < 32) {
    read_argv(i+1, arg, 31)
    parse(arg, flags, 31, num, 31)
    g_TimeSet[i][0] = str_to_num(num)
    g_TimeSet[i][1] = read_flags(flags)
    i++
  }
  g_TimeSet[i][0] = 0
  return PLUGIN_HANDLED
}

public timeRemain(taskId) {
  new gmtm = get_timeleft()
  if(gmtm == 0 && get_cvar_num("mp_timelimit") > 0 && get_playersnum() == 0) {
    changeMap()
  }
  new tmlf = g_Switch ? --g_CountDown : gmtm
  new stimel[12]
  format(stimel, 11, "%02d:%02d", gmtm / 60, gmtm % 60)
  set_cvar_string("amx_timeleft", stimel)
  new Float:gameTime = get_gametime()
  new Float:offset = gameTime - g_gameTime
  offset = offset - floatround(offset, floatround_floor)
  offset = floatround(offset*10.0, floatround_floor) / 10.0
  set_task(1.0 - offset, "timeRemain", taskId)
  if(g_Switch && gmtm > g_Switch) {
    remove_task(34543)
    g_Switch = 0
    gameTime = get_gametime()
    offset = gameTime - g_gameTime
    offset = offset - floatround(offset, floatround_floor)
    offset = floatround(offset*10.0, floatround_floor) / 10.0
    set_task(1.0 - offset, "timeRemain", 8648458)
    return
  }
  if(tmlf > 0 && g_LastTime != tmlf) {
    g_LastTime = tmlf
    new tm_set = findDispFormat(tmlf)
    if(tm_set != -1) {
      new flags = g_TimeSet[tm_set][1]
      new arg[128]
      if(flags & 1) {
        new p[32], pnum
        get_players(p, pnum, "c")
        for(pnum--; pnum >= 0; pnum--) {
          setTimeText(arg, 127, tmlf, p[pnum])
          if(flags & 16)
            set_hudmessage(255, 255, 255, -1.0, 0.85, 0, 0.0, 1.1, 0.1, 0.5, 1)
          else
            set_hudmessage(255, 255, 255, -1.0, 0.85, 0, 0.0, 3.0, 0.0, 0.5, 1)
          show_hudmessage(p[pnum], arg)
        }
      }
      if(flags & 2) {
        setTimeVoice(arg, 127, flags, tmlf)
        client_cmd(0, arg)
      }
    }
  }
}

getNextMapName(szArg[],iMax) {
  new len = get_cvar_string("amx_nextmap", szArg, iMax)
  if(is_map_valid(szArg)) return len
  len = copy(szArg, iMax, g_nextMap)
  set_cvar_string("amx_nextmap", g_nextMap)
  return len
}

public delayedChange(param[])
  server_cmd("changelevel %s", param)

public changeMap() {
  new string[32]
  new len = getNextMapName(string, 31) + 1
  new modName[8]
  get_modname(modName, 7)
  if(!equal(modName, "zp")) {
    message_begin(MSG_ALL, SVC_INTERMISSION)
    message_end()
  }
  set_task(1.0, "delayedChange", 0, string, len)
}
